@@ -209,6 +209,10 @@ static void get_name_property(
       *ret_name = NoName;
     }
   }
+  else
+  {
+    *ret_name = NoName;
+  }
 
   return;
 }
@@ -414,22 +418,26 @@ void setup_style_and_decor(
 
 #ifdef USEDECOR
   /* search for a UseDecor tag in the style */
-  tmp_win->decor = NULL;
-  if (SGET_DECOR_NAME(*pstyle) != NULL)
+  if (tmp_win->decor == NULL)
   {
-    FvwmDecor *decor = &Scr.DefaultDecor;
-
-    for (; decor; decor = decor->next)
+    if (SGET_DECOR_NAME(*pstyle) != NULL)
     {
-      if (StrEquals(SGET_DECOR_NAME(*pstyle), decor->tag))
+      FvwmDecor *decor = &Scr.DefaultDecor;
+
+      for (; decor; decor = decor->next)
       {
-	tmp_win->decor = decor;
-	break;
+	if (StrEquals(SGET_DECOR_NAME(*pstyle), decor->tag))
+	{
+	  tmp_win->decor = decor;
+	  break;
+	}
       }
     }
+    if (tmp_win->decor == NULL)
+    {
+      tmp_win->decor = &Scr.DefaultDecor;
+    }
   }
-  if (tmp_win->decor == NULL)
-    tmp_win->decor = &Scr.DefaultDecor;
 #endif
 
   GetMwmHints(tmp_win);
@@ -459,17 +467,23 @@ void setup_style_and_decor(
 
 void setup_icon_boxes(FvwmWindow *tmp_win, window_style *pstyle)
 {
+  icon_boxes *ib;
+
   /* copy iconboxes ptr (if any) */
   if (SHAS_ICON_BOXES(&pstyle->flags))
   {
     tmp_win->IconBoxes = SGET_ICON_BOXES(*pstyle);
-    tmp_win->IconBoxes->use_count++;
+    for (ib = tmp_win->IconBoxes; ib; ib = ib->next)
+    {
+      ib->use_count++;
+    }
   }
   else
+  {
     tmp_win->IconBoxes = NULL;
+  }
 }
 
-
 void destroy_icon_boxes(FvwmWindow *tmp_win)
 {
   if (tmp_win->IconBoxes)
@@ -1283,6 +1297,7 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   sflags = SGET_FLAGS_POINTER(style);
   SET_TRANSIENT(
     tmp_win, !!XGetTransientForHint(dpy, tmp_win->w, &tmp_win->transientfor));
+  tmp_win->decor = NULL;
   setup_style_and_decor(tmp_win, &style, &buttons);
 
   /****** fonts ******/
@@ -2061,7 +2076,8 @@ void destroy_window(FvwmWindow *tmp_win)
  *  Puts windows back where they were before fvwm took over
  *
  ************************************************************************/
-void RestoreWithdrawnLocation(FvwmWindow *tmp, Bool restart, Window parent)
+void RestoreWithdrawnLocation(
+  FvwmWindow *tmp, Bool is_restart_or_recapture, Window parent)
 {
   int w2,h2;
   unsigned int mask;
@@ -2104,7 +2120,7 @@ void RestoreWithdrawnLocation(FvwmWindow *tmp, Bool restart, Window parent)
    * half off the screen. (RN)
    */
 
-  if (!restart)
+  if (!is_restart_or_recapture)
   {
     /* Don't mess with it if its partially on the screen now */
     if (unshaded_g.x < 0 || unshaded_g.y < 0 ||
@@ -2113,16 +2129,16 @@ void RestoreWithdrawnLocation(FvwmWindow *tmp, Bool restart, Window parent)
     {
       w2 = (unshaded_g.width >> 1);
       h2 = (unshaded_g.height >> 1);
-      if (( xwc.x < -w2) || (xwc.x > (Scr.MyDisplayWidth - w2)))
+      if ( xwc.x < -w2 || xwc.x > Scr.MyDisplayWidth - w2)
       {
 	xwc.x = xwc.x % Scr.MyDisplayWidth;
-	if ( xwc.x < -w2 )
+	if (xwc.x < -w2)
 	  xwc.x += Scr.MyDisplayWidth;
       }
-      if ((xwc.y < -h2) || (xwc.y > (Scr.MyDisplayHeight  -h2)))
+      if (xwc.y < -h2 || xwc.y > Scr.MyDisplayHeight - h2)
       {
 	xwc.y = xwc.y % Scr.MyDisplayHeight;
-	if ( xwc.y < -h2 )
+	if (xwc.y < -h2)
 	  xwc.y += Scr.MyDisplayHeight;
       }
     }
@@ -2130,7 +2146,7 @@ void RestoreWithdrawnLocation(FvwmWindow *tmp, Bool restart, Window parent)
   XReparentWindow(
     dpy, tmp->w, (parent == None) ? Scr.Root : parent, xwc.x, xwc.y);
 
-  if(IS_ICONIFIED(tmp) && !IS_ICON_SUPPRESSED(tmp))
+  if (IS_ICONIFIED(tmp) && !IS_ICON_SUPPRESSED(tmp))
   {
     if (tmp->icon_w)
       XUnmapWindow(dpy, tmp->icon_w);
@@ -2138,7 +2154,10 @@ void RestoreWithdrawnLocation(FvwmWindow *tmp, Bool restart, Window parent)
       XUnmapWindow(dpy, tmp->icon_pixmap_w);
   }
 
-  XConfigureWindow (dpy, tmp->w, mask, &xwc);
-  if(!restart)
+  XConfigureWindow(dpy, tmp->w, mask, &xwc);
+  if (!is_restart_or_recapture)
+  {
+    /* must be initial capture */
     XSync(dpy,0);
+  }
 }
