@@ -44,6 +44,7 @@
 #include "misc.h"
 #include "screen.h"
 #include "defaults.h"
+#include "geometry.h"
 #include "borders.h"
 #include "builtins.h"
 #include "icons.h"
@@ -150,6 +151,37 @@ static void DrawLinePattern(
   }
 }
 
+/* used by DrawButton() */
+static void clip_button_pixmap(
+  int *ox, int *oy, int *cw, int *ch, int*cx, int *cy, XRectangle *rclip)
+{
+  if (rclip)
+  {
+    if (rclip->x > *cx)
+    {
+      *ox = (rclip->x - *cx);
+      *cx += *ox;
+      *cw -= *ox;
+    }
+    if (*cx + *cw > rclip->x + rclip->width)
+    {
+      *cw = rclip->x - *cx + rclip->width;
+    }
+    if (rclip->y > *cy)
+    {
+      *oy = (rclip->y - *cy);
+      *cy += *oy;
+      *ch -= *oy;
+    }
+    if (*cy + *ch > rclip->y + rclip->height)
+    {
+      *ch = rclip->y - *cy + rclip->height;
+    }
+  }
+
+  return;
+}
+
 /****************************************************************************
  *
  *  Redraws buttons (veliaa@rpi.edu)
@@ -158,140 +190,197 @@ static void DrawLinePattern(
 static void DrawButton(FvwmWindow *t, Window win, int w, int h,
 		       DecorFace *df, GC ReliefGC, GC ShadowGC,
 		       Boolean inverted, mwm_flags stateflags,
-		       int left1right0)
+		       int left1right0, XRectangle *rclip)
 {
   register DecorFaceType type = DFS_FACE_TYPE(df->style);
   Picture *p;
   int border = 0;
   int width, height, x, y;
 
+  /* Note: it is assumed that ReliefGC and ShadowGC are already clipped with
+   * the rclip rectangle when this function is called. */
+
   switch (type)
   {
-    case SimpleButton:
-      break;
+  case SimpleButton:
+    break;
 
-    case SolidButton:
-      XSetWindowBackground(dpy, win, df->u.back);
-      flush_expose(win);
-      XClearWindow(dpy,win);
-      break;
-
-    case VectorButton:
-    case DefaultVectorButton:
-      if(HAS_MWM_BUTTONS(t) &&
-	 ((stateflags & MWM_DECOR_MAXIMIZE && IS_MAXIMIZED(t)) ||
-	  (stateflags & MWM_DECOR_SHADE && IS_SHADED(t)) ||
-	  (stateflags & MWM_DECOR_STICK && IS_STICKY(t))))
-	DrawLinePattern(win, ShadowGC, ReliefGC, &df->u.vector, w, h);
-      else
-	DrawLinePattern(win, ReliefGC, ShadowGC, &df->u.vector, w, h);
-      break;
-
-#ifdef MINI_ICONS
-    case MiniIconButton:
-    case PixmapButton:
-    case TiledPixmapButton:
-      if (type == PixmapButton || type == TiledPixmapButton)
-	p = df->u.p;
-      else
-      {
-	if (!t->mini_icon)
-	  break;
-	p = t->mini_icon;
-      }
-#else
-    case PixmapButton:
-    case TiledPixmapButton:
-      p = df->u.p;
-#endif /* MINI_ICONS */
-      if (DFS_BUTTON_RELIEF(df->style) == DFS_BUTTON_IS_FLAT)
-	border = 0;
-      else
-	border = HAS_MWM_BORDER(t) ? 1 : 2;
-      width = w - border * 2;
-      height = h - border * 2;
-
-      x = border;
-      y = border;
-      if (type != TiledPixmapButton)
-      {
-	if (DFS_H_JUSTIFICATION(df->style) == JUST_RIGHT)
-	  x += (int)(width - p->width);
-	else if (DFS_H_JUSTIFICATION(df->style) == JUST_CENTER)
-	  /* round up for left buttons, down for right buttons */
-	  x += (int)(width - p->width + left1right0) / 2;
-	if (DFS_V_JUSTIFICATION(df->style) == JUST_BOTTOM)
-	  y += (int)(height - p->height);
-	else if (DFS_V_JUSTIFICATION(df->style) == JUST_CENTER)
-	  /* round up */
-	  y += (int)(height - p->height + 1) / 2;
-	if (x < border)
-	  x = border;
-	if (y < border)
-	  y = border;
-	if (width > p->width)
-	  width = p->width;
-	if (height > p->height)
-	  height = p->height;
-	if (width > w - x - border)
-	  width = w - x - border;
-	if (height > h - y - border)
-	  height = h - y - border;
-      }
-
-      XSetClipMask(dpy, Scr.TransMaskGC, p->mask);
-      if (type != TiledPixmapButton)
-      {
-	XSetClipOrigin(dpy, Scr.TransMaskGC, x, y);
-	XCopyArea(dpy, p->picture, win, Scr.TransMaskGC,
-		  0, 0, width, height, x, y);
-      }
-      else
-      {
-	int xi;
-	int yi;
-
-	for (yi = border; yi < height; yi += p->height)
-	{
-	  for (xi = border; xi < width; xi += p->width)
-	  {
-	    int lw = width - xi - 0*p->width;
-	    int lh = height - yi - 0*p->height;
-
-	    if (lw > p->width)
-	      lw = p->width;
-	    if (lh > p->height)
-	      lh = p->height;
-	    XSetClipOrigin(dpy, Scr.TransMaskGC, xi, yi);
-	    XCopyArea(dpy, p->picture, win, Scr.TransMaskGC,
-		      0, 0, lw, lh, xi, yi);
-	  }
-	}
-      }
-      break;
-
-    case GradientButton:
+  case SolidButton:
+    XSetWindowBackground(dpy, win, df->u.back);
+    if (rclip)
+    {
+      XClearArea(
+	dpy, win, rclip->x, rclip->y, rclip->width, rclip->height,
+	False);
+    }
+    else
     {
-      unsigned int g_width;
-      unsigned int g_height;
-
       flush_expose(win);
-      XSetClipMask(dpy, Scr.TransMaskGC, None);
-      /* find out the size the pixmap should be */
-      CalculateGradientDimensions(
-	dpy, win, df->u.grad.npixels, df->u.grad.gradient_type,
-	&g_width, &g_height);
-      /* draw the gradient directly into the window */
-      CreateGradientPixmap(
-	dpy, win, Scr.TransMaskGC, df->u.grad.gradient_type, g_width, g_height,
-	df->u.grad.npixels, df->u.grad.pixels, win, 0, 0, w, h);
+      XClearWindow(dpy, win);
     }
     break;
 
-    default:
-      fvwm_msg(ERR,"DrawButton","unknown button type");
-      break;
+  case VectorButton:
+  case DefaultVectorButton:
+    if(HAS_MWM_BUTTONS(t) &&
+       ((stateflags & MWM_DECOR_MAXIMIZE && IS_MAXIMIZED(t)) ||
+	(stateflags & MWM_DECOR_SHADE && IS_SHADED(t)) ||
+	(stateflags & MWM_DECOR_STICK && IS_STICKY(t))))
+    {
+      DrawLinePattern(win, ShadowGC, ReliefGC, &df->u.vector, w, h);
     }
+    else
+    {
+      DrawLinePattern(win, ReliefGC, ShadowGC, &df->u.vector, w, h);
+    }
+    break;
+
+#ifdef MINI_ICONS
+  case MiniIconButton:
+  case PixmapButton:
+  case TiledPixmapButton:
+    if (type == PixmapButton || type == TiledPixmapButton)
+      p = df->u.p;
+    else
+    {
+      if (!t->mini_icon)
+	break;
+      p = t->mini_icon;
+    }
+#else
+  case PixmapButton:
+  case TiledPixmapButton:
+    p = df->u.p;
+#endif /* MINI_ICONS */
+    if (DFS_BUTTON_RELIEF(df->style) == DFS_BUTTON_IS_FLAT)
+      border = 0;
+    else
+      border = HAS_MWM_BORDER(t) ? 1 : 2;
+    width = w - border * 2;
+    height = h - border * 2;
+
+    x = border;
+    y = border;
+    if (type != TiledPixmapButton)
+    {
+      if (DFS_H_JUSTIFICATION(df->style) == JUST_RIGHT)
+	x += (int)(width - p->width);
+      else if (DFS_H_JUSTIFICATION(df->style) == JUST_CENTER)
+	/* round up for left buttons, down for right buttons */
+	x += (int)(width - p->width + left1right0) / 2;
+      if (DFS_V_JUSTIFICATION(df->style) == JUST_BOTTOM)
+	y += (int)(height - p->height);
+      else if (DFS_V_JUSTIFICATION(df->style) == JUST_CENTER)
+	/* round up */
+	y += (int)(height - p->height + 1) / 2;
+      if (x < border)
+	x = border;
+      if (y < border)
+	y = border;
+      if (width > p->width)
+	width = p->width;
+      if (height > p->height)
+	height = p->height;
+      if (width > w - x - border)
+	width = w - x - border;
+      if (height > h - y - border)
+	height = h - y - border;
+    }
+
+    XSetClipMask(dpy, Scr.TransMaskGC, p->mask);
+    XSetClipOrigin(dpy, Scr.TransMaskGC, x, y);
+    if (type != TiledPixmapButton)
+    {
+      int cx = x;
+      int cy = y;
+      int cw = width;
+      int ch = height;
+      int ox = 0;
+      int oy = 0;
+
+      clip_button_pixmap(&ox, &oy, &cw, &ch, &cx, &cy, rclip);
+      if (cw > 0 && ch > 0)
+      {
+	XCopyArea(
+	  dpy, p->picture, win, Scr.TransMaskGC, ox, oy, cw, ch, cx, cy);
+      }
+    }
+    else
+    {
+      int xi;
+      int yi;
+
+      for (yi = border; yi < height; yi += p->height)
+      {
+	for (xi = border; xi < width; xi += p->width)
+	{
+	  int lw = width - xi;
+	  int lh = height - yi;
+	  int cx;
+	  int cy;
+	  int cw;
+	  int ch;
+	  int ox;
+	  int oy;
+
+	  if (lw > p->width)
+	    lw = p->width;
+	  if (lh > p->height)
+	    lh = p->height;
+	  if (rclip)
+	  {
+	    if (xi + lw <= rclip->x || xi >= rclip->x + rclip->width ||
+		yi + lh <= rclip->y || yi >= rclip->y + rclip->height)
+	    {
+	      continue;
+	    }
+	  }
+	  cx = xi;
+	  cy = yi;
+	  cw = lw;
+	  ch = lh;
+	  ox = 0;
+	  oy = 0;
+	  clip_button_pixmap(&ox, &oy, &cw, &ch, &cx, &cy, rclip);
+	  if (cw > 0 && ch > 0)
+	  {
+	    XSetClipOrigin(dpy, Scr.TransMaskGC, xi, yi);
+	    XCopyArea(dpy, p->picture, win, Scr.TransMaskGC,
+		      ox, oy, cw, ch, cx, cy);
+	  }
+	}
+      }
+    }
+    XSetClipMask(dpy, Scr.TransMaskGC, None);
+    break;
+
+  case GradientButton:
+  {
+    unsigned int g_width;
+    unsigned int g_height;
+
+    if (!rclip)
+    {
+      flush_expose(win);
+    }
+    XSetClipMask(dpy, Scr.TransMaskGC, None);
+    /* find out the size the pixmap should be */
+    CalculateGradientDimensions(
+      dpy, win, df->u.grad.npixels, df->u.grad.gradient_type,
+      &g_width, &g_height);
+    /* draw the gradient directly into the window */
+    CreateGradientPixmap(
+      dpy, win, Scr.TransMaskGC, df->u.grad.gradient_type, g_width, g_height,
+      df->u.grad.npixels, df->u.grad.pixels, win, 0, 0, w, h, rclip);
+  }
+  break;
+
+  default:
+    fvwm_msg(ERR,"DrawButton","unknown button type");
+    break;
+  }
+
+  return;
 }
 
 /* rules to get button state */
@@ -368,7 +457,7 @@ static void draw_frame_relief(
  ****************************************************************************/
 static void RedrawBorder(
   common_decorations_type *cd, FvwmWindow *t, Bool has_focus, int force,
-  Window expose_win)
+  Window expose_win, XRectangle *rclip)
 {
   static GC tgc = None;
   int i;
@@ -376,6 +465,7 @@ static void RedrawBorder(
   GC sgc;
   DecorFaceStyle *borderstyle;
   Bool is_reversed = False;
+  Bool is_clipped = False;
   int w_dout;
   int w_hiout;
   int w_trout;
@@ -399,7 +489,9 @@ static void RedrawBorder(
    * it will show through. The entire border is redrawn so flush all exposes
    * up to this point.
    */
+#if 0
   flush_expose(t->decor_w);
+#endif
 
   if (t->boundary_width < 2)
   {
@@ -449,7 +541,18 @@ static void RedrawBorder(
   if (PressedW == None &&
       (DFS_HAS_NO_INSET(*borderstyle) ||
        DFS_HAS_HIDDEN_HANDLES(*borderstyle)))
-    XClearWindow(dpy, t->decor_w);
+  {
+    if (rclip)
+    {
+      XClearArea(
+	dpy, t->decor_w, rclip->x, rclip->y, rclip->width, rclip->height,
+	False);
+    }
+    else
+    {
+      XClearWindow(dpy, t->decor_w);
+    }
+  }
 
   /*
    * Nothing to do if flat
@@ -573,6 +676,12 @@ static void RedrawBorder(
     sum--;
   }
   w_c = t->boundary_width - sum;
+  if (rclip)
+  {
+    XSetClipRectangles(dpy, rgc, 0, 0, rclip, 1, Unsorted);
+    XSetClipRectangles(dpy, sgc, 0, 0, rclip, 1, Unsorted);
+    is_clipped = True;
+  }
   draw_frame_relief(
     t, rgc, sgc, tgc, sgc,
     w_dout, w_hiout, w_trout, w_c, w_trin, w_shin, w_din);
@@ -713,11 +822,14 @@ static void RedrawBorder(
     else if (PressedW == t->sides[1])
     {
       /* right */
-      r.x = t->frame_g.width - t->boundary_width;
-      r.y = t->visual_corner_width;
-      r.width = t->boundary_width - 1;
-      r.height = t->frame_g.height - 2 * t->visual_corner_width;
-      is_pressed = True;
+      if (!IS_SHADED(t))
+      {
+	r.x = t->frame_g.width - t->boundary_width;
+	r.y = t->visual_corner_width;
+	r.width = t->boundary_width - 1;
+	r.height = t->frame_g.height - 2 * t->visual_corner_width;
+	is_pressed = True;
+      }
     }
     else if (PressedW == t->sides[2])
     {
@@ -731,11 +843,14 @@ static void RedrawBorder(
     else if (PressedW == t->sides[3])
     {
       /* left */
-      r.x = 1;
-      r.y = t->visual_corner_width;
-      r.width = t->boundary_width - 1;
-      r.height = t->frame_g.height - 2 * t->visual_corner_width;
-      is_pressed = True;
+      if (!IS_SHADED(t))
+      {
+	r.x = 1;
+	r.y = t->visual_corner_width;
+	r.width = t->boundary_width - 1;
+	r.height = t->frame_g.height - 2 * t->visual_corner_width;
+	is_pressed = True;
+      }
     }
     else if (PressedW == t->corners[0])
     {
@@ -785,15 +900,30 @@ static void RedrawBorder(
 
     if (is_pressed == True)
     {
-      XSetClipRectangles(dpy, rgc, 0, 0, &r, 1, Unsorted);
-      XSetClipRectangles(dpy, sgc, 0, 0, &r, 1, Unsorted);
-      draw_frame_relief(
-	t, sgc, rgc, tgc, sgc,
-	w_dout, w_hiout, w_trout, w_c, w_trin, w_shin, w_din);
-      XSetClipMask(dpy, sgc, None);
-      XSetClipMask(dpy, rgc, None);
+      Bool is_not_empty = True;
+
+      if (rclip)
+      {
+	is_not_empty = intersect_xrectangles(&r, rclip);
+      }
+      if (is_not_empty)
+      {
+	XSetClipRectangles(dpy, rgc, 0, 0, &r, 1, Unsorted);
+	XSetClipRectangles(dpy, sgc, 0, 0, &r, 1, Unsorted);
+	draw_frame_relief(
+	  t, sgc, rgc, tgc, sgc,
+	  w_dout, w_hiout, w_trout, w_c, w_trin, w_shin, w_din);
+	is_clipped = True;
+      }
     }
   }
+  if (is_clipped)
+  {
+    XSetClipMask(dpy, rgc, None);
+    XSetClipMask(dpy, sgc, None);
+  }
+
+  return;
 }
 
 /****************************************************************************
@@ -803,10 +933,14 @@ static void RedrawBorder(
  ****************************************************************************/
 static void RedrawButtons(
   common_decorations_type *cd, FvwmWindow *t, Bool has_focus, int force,
-  Window expose_win)
+  Window expose_win, XRectangle *rclip)
 {
   int i;
 
+  /* Note: the rclip rectangle is not used in this function. Buttons are usually
+   * so small that it makes not much sense to limit drawing to a clip rectangle.
+   */
+
   /*
    * draw buttons
    */
@@ -850,7 +984,7 @@ static void RedrawButtons(
 	  {
 	    DrawButton(t, t->button_w[i], t->title_g.height, t->title_g.height,
 		       tsdf, cd->relief_gc, cd->shadow_gc, inverted,
-		       TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i])), 1);
+		       TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i])), 1, NULL);
 	  }
 	}
 #ifdef MULTISTYLE
@@ -859,7 +993,7 @@ static void RedrawButtons(
 	{
 	  DrawButton(t, t->button_w[i], t->title_g.height, t->title_g.height,
 		     df, cd->relief_gc, cd->shadow_gc, inverted,
-		     TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i])), 1);
+		     TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i])), 1, NULL);
 	}
 
 	{
@@ -892,7 +1026,8 @@ static void RedrawButtons(
  *  Redraws just the title bar
  *
  ****************************************************************************/
-void RedrawTitle(common_decorations_type *cd, FvwmWindow *t, Bool has_focus)
+static void RedrawTitle(
+  common_decorations_type *cd, FvwmWindow *t, Bool has_focus, XRectangle *rclip)
 {
   int hor_off;
   int w;
@@ -902,6 +1037,7 @@ void RedrawTitle(common_decorations_type *cd, FvwmWindow *t, Bool has_focus)
   GC rgc = cd->relief_gc;
   GC sgc = cd->shadow_gc;
   Bool reverse = False;
+  Bool is_clipped = False;
   Bool toggled =
     (HAS_MWM_BUTTONS(t) &&
      ((TB_HAS_MWM_DECOR_MAXIMIZE(GetDecor(t, titlebar)) && IS_MAXIMIZED(t))||
@@ -916,7 +1052,9 @@ void RedrawTitle(common_decorations_type *cd, FvwmWindow *t, Bool has_focus)
     sgc = rgc;
     rgc = tgc;
   }
+#if 0
   flush_expose(t->title_w);
+#endif
 
   if (t->name != (char *)NULL)
   {
@@ -966,7 +1104,20 @@ void RedrawTitle(common_decorations_type *cd, FvwmWindow *t, Bool has_focus)
       XSetWindowBackgroundPixmap(dpy, t->title_w, df->u.p->picture);
     }
   }
-  XClearWindow(dpy, t->title_w);
+  if (rclip)
+  {
+    XClearArea(
+      dpy, t->title_w, rclip->x, rclip->y, rclip->width, rclip->height,
+      False);
+    XSetClipRectangles(dpy, rgc, 0, 0, rclip, 1, Unsorted);
+    XSetClipRectangles(dpy, sgc, 0, 0, rclip, 1, Unsorted);
+    XSetClipRectangles(dpy, Scr.TitleGC, 0, 0, rclip, 1, Unsorted);
+    is_clipped = True;
+  }
+  else
+  {
+    XClearWindow(dpy, t->title_w);
+  }
 
   /*
    * draw title
@@ -1010,7 +1161,7 @@ void RedrawTitle(common_decorations_type *cd, FvwmWindow *t, Bool has_focus)
       {
 	DrawButton(
 	  t, t->title_w, t->title_g.width, t->title_g.height, df, sgc,
-	  rgc, True, 0, 1);
+	  rgc, True, 0, 1, rclip);
       }
     }
     else
@@ -1021,14 +1172,12 @@ void RedrawTitle(common_decorations_type *cd, FvwmWindow *t, Bool has_focus)
       {
 	DrawButton(
 	  t, t->title_w, t->title_g.width, t->title_g.height, df, rgc,
-	  sgc, False, 0, 1);
+	  sgc, False, 0, 1, rclip);
       }
     }
-
     /*
      * draw title relief
      */
-
     switch (DFS_BUTTON_RELIEF(*tb_style))
     {
     case DFS_BUTTON_IS_SUNK:
@@ -1074,6 +1223,15 @@ void RedrawTitle(common_decorations_type *cd, FvwmWindow *t, Bool has_focus)
 	t->title_g.width - hor_off - w - 11, 1, sgc, rgc, 1);
     }
   }
+
+  if (is_clipped)
+  {
+    XSetClipMask(dpy, rgc, None);
+    XSetClipMask(dpy, sgc, None);
+    XSetClipMask(dpy, Scr.TitleGC, None);
+  }
+
+  return;
 }
 
 
@@ -1187,7 +1345,8 @@ void SetupTitleBar(FvwmWindow *tmp_win, int w, int h)
 
 static void get_common_decorations(
   common_decorations_type *cd, FvwmWindow *t, draw_window_parts draw_parts,
-  Bool has_focus, int force, Window expose_win, Bool is_border)
+  Bool has_focus, int force, Window expose_win, Bool is_border,
+  Bool do_change_gcs)
 {
   color_quad *draw_colors;
 
@@ -1200,8 +1359,10 @@ static void get_common_decorations(
       cd->texture_pixmap = GetDecor(t, BorderStyle.active.u.p->picture);
     }
     cd->back_pixmap= Scr.gray_pixmap;
-    if (is_border) draw_colors = &(t->border_hicolors);
-    else draw_colors = &(t->hicolors);
+    if (is_border)
+      draw_colors = &(t->border_hicolors);
+    else
+      draw_colors = &(t->hicolors);
   }
   else
   {
@@ -1219,13 +1380,16 @@ static void get_common_decorations(
   }
   cd->fore_color = draw_colors->fore;
   cd->back_color = draw_colors->back;
-  Globalgcv.foreground = draw_colors->hilight;
-  Globalgcm = GCForeground;
-  XChangeGC(dpy,Scr.ScratchGC1,Globalgcm,&Globalgcv);
-  cd->relief_gc = Scr.ScratchGC1;
-  Globalgcv.foreground = draw_colors->shadow;
-  XChangeGC(dpy, Scr.ScratchGC2, Globalgcm, &Globalgcv);
-  cd->shadow_gc = Scr.ScratchGC2;
+  if (do_change_gcs)
+  {
+    Globalgcv.foreground = draw_colors->hilight;
+    Globalgcm = GCForeground;
+    XChangeGC(dpy,Scr.ScratchGC1,Globalgcm,&Globalgcv);
+    cd->relief_gc = Scr.ScratchGC1;
+    Globalgcv.foreground = draw_colors->shadow;
+    XChangeGC(dpy, Scr.ScratchGC2, Globalgcm, &Globalgcv);
+    cd->shadow_gc = Scr.ScratchGC2;
+  }
 
   /* MWMBorder style means thin 3d effects */
   cd->relief_width = (HAS_MWM_BORDER(t) ? 1 : 2);
@@ -1264,14 +1428,17 @@ static void get_common_decorations(
   }
 }
 
-void DrawDecorations(
+void draw_clipped_decorations(
   FvwmWindow *t, draw_window_parts draw_parts, Bool has_focus, int force,
-  Window expose_win)
+  Window expose_win, XRectangle *rclip)
 {
   common_decorations_type cd;
   Bool is_frame_redraw_allowed = False;
   Bool is_button_redraw_allowed = False;
   Bool is_title_redraw_allowed = False;
+  Bool do_redraw_title = False;
+  Bool do_redraw_buttons = False;
+  Bool do_change_gcs = False;
 
   if (!t)
     return;
@@ -1284,14 +1451,9 @@ void DrawDecorations(
   }
   else if (expose_win == t->title_w)
     is_title_redraw_allowed = True;
-  else if (expose_win == t->frame || expose_win == t->decor_w
-/* sides and corners are InputOnly and incapable of triggering Expose events
-           || (HAS_BORDER(t) &&
-            (expose_win == t->sides[0] || expose_win == t->corners[0] ||
-             expose_win == t->sides[1] || expose_win == t->corners[1] ||
-             expose_win == t->sides[2] || expose_win == t->corners[2] ||
-             expose_win == t->sides[3] || expose_win == t->corners[3])) */
-           )
+  else if (expose_win == t->frame || expose_win == t->decor_w)
+    /* sides and corners are InputOnly and incapable of triggering Expose events
+     */
   {
     is_frame_redraw_allowed = True;
   }
@@ -1341,34 +1503,64 @@ void DrawDecorations(
     return;
   }
 
+  /* calculate some values and flags */
+  if ((draw_parts & DRAW_TITLE) && HAS_TITLE(t) && is_title_redraw_allowed)
+  {
+    do_redraw_title = True;
+    do_change_gcs = True;
+  }
+  if ((draw_parts & DRAW_BUTTONS) && HAS_TITLE(t) && is_button_redraw_allowed)
+  {
+    do_redraw_buttons = True;
+    do_change_gcs = True;
+  }
   get_common_decorations(
-    &cd, t, draw_parts, has_focus, force, expose_win, FALSE);
+    &cd, t, draw_parts, has_focus, force, expose_win, False, do_change_gcs);
 
+  /* redraw */
   if (cd.flags.has_color_changed && HAS_TITLE(t))
   {
-    change_window_background(
-      t->title_w, cd.notex_valuemask, &cd.notex_attributes);
+    if (!do_redraw_title || !rclip)
+    {
+      change_window_background(
+	t->title_w, cd.notex_valuemask, &cd.notex_attributes);
+    }
+  }
+  if (do_redraw_buttons)
+  {
+    RedrawButtons(&cd, t, has_focus, force, expose_win, rclip);
+  }
+  if (do_redraw_title)
+  {
+    RedrawTitle(&cd, t, has_focus, rclip);
   }
-
-  if ((draw_parts & DRAW_BUTTONS) && HAS_TITLE(t) && is_button_redraw_allowed)
-    RedrawButtons(&cd, t, has_focus, force, expose_win);
-  if ((draw_parts & DRAW_TITLE) && HAS_TITLE(t) && is_title_redraw_allowed)
-    RedrawTitle(&cd, t, has_focus);
-  get_common_decorations(
-    &cd, t, draw_parts, has_focus, force, expose_win, TRUE);
   if (cd.flags.has_color_changed ||
       ((draw_parts & DRAW_FRAME) && (is_frame_redraw_allowed)))
   {
     get_common_decorations(
-      &cd, t, draw_parts, has_focus, force, expose_win, TRUE);
-    change_window_background(t->decor_w, cd.valuemask, &cd.attributes);
-    RedrawBorder(&cd, t, has_focus, force, expose_win);
+      &cd, t, draw_parts, has_focus, force, expose_win, True, True);
+    if (!rclip || !IS_WINDOW_BORDER_DRAWN(t))
+    {
+      change_window_background(t->decor_w, cd.valuemask, &cd.attributes);
+      SET_WINDOW_BORDER_DRAWN(t, 1);
+      rclip = NULL;
+    }
+    RedrawBorder(&cd, t, has_focus, force, expose_win, rclip);
   }
 
   /* Sync to make the change look fast! */
   XSync(dpy,0);
 }
 
+void DrawDecorations(
+  FvwmWindow *t, draw_window_parts draw_parts, Bool has_focus, int force,
+  Window expose_win)
+{
+  draw_clipped_decorations(t, draw_parts, has_focus, force, expose_win, NULL);
+
+  return;
+}
+
 
 /***********************************************************************
  *
@@ -1545,6 +1737,20 @@ void SetupFrame(
 	/* This reduces flickering */
 	XSync(dpy, 0);
       }
+      else if (HAS_BOTTOM_TITLE(tmp_win))
+      {
+	rectangle big_g;
+
+	big_g = (IS_MAXIMIZED(tmp_win)) ? tmp_win->max_g : tmp_win->normal_g;
+	get_relative_geometry(&big_g, &big_g);
+	XMoveWindow(dpy, tmp_win->w, 0, 1 - big_g.height +
+		    2 * tmp_win->boundary_width + tmp_win->title_g.height);
+      }
+      else
+      {
+	XMoveWindow(dpy, tmp_win->w, 0, 0);
+      }
+
       break;
     case 3:
       XMoveResizeWindow(dpy, tmp_win->decor_w, dx, dy, w, h);
@@ -1775,21 +1981,23 @@ void SetShape(FvwmWindow *tmp_win, int w)
 	tmp_win->title_top_height + tmp_win->boundary_width, tmp_win->w,
 	ShapeBounding, ShapeSet);
       if (tmp_win->title_w)
-	{
-	  /* windows w/ titles */
-	  rect.x = tmp_win->boundary_width;
-	  rect.y = tmp_win->title_g.y;
-	  rect.width = w - 2*tmp_win->boundary_width;
-	  rect.height = tmp_win->title_g.height;
+      {
+	/* windows w/ titles */
+	rect.x = tmp_win->boundary_width;
+	rect.y = tmp_win->title_g.y;
+	rect.width = w - 2*tmp_win->boundary_width;
+	rect.height = tmp_win->title_g.height;
 
-
-	  XShapeCombineRectangles(dpy,tmp_win->frame,ShapeBounding,
-				  0,0,&rect,1,ShapeUnion,Unsorted);
-	}
+	XShapeCombineRectangles(
+	  dpy,tmp_win->frame,ShapeBounding,0,0,&rect,1,ShapeUnion,Unsorted);
+      }
     }
     else
-      XShapeCombineMask (dpy, tmp_win->frame, ShapeBounding,
-			 0, 0, None, ShapeSet);
+    {
+      /* unset shape */
+      XShapeCombineMask(
+	dpy, tmp_win->frame, ShapeBounding, 0, 0, None, ShapeSet);
+    }
   }
 #endif
 }
