@@ -24,6 +24,7 @@ typedef enum
   DRAW_ALL      = 0x7
 } draw_window_parts;
 
+int ButtonPosition(int context, FvwmWindow * t);
 void SetupTitleBar(FvwmWindow *tmp_win, int w, int h);
 void SetupFrame(
   FvwmWindow *tmp_win, int x, int y, int w, int h, Bool sendEvent);
@@ -34,6 +35,9 @@ void set_decor_gravity(
 void SetShape(FvwmWindow *, int);
 void cmd_button_state(F_CMD_ARGS);
 void SetBorderStyle(F_CMD_ARGS);
+void draw_clipped_decorations(
+  FvwmWindow *t, draw_window_parts draw_parts, Bool has_focus, int force,
+  Window expose_win, XRectangle *rclip);
 void DrawDecorations(
   FvwmWindow *t, draw_window_parts draw_parts, Bool has_focus, int force,
   Window expose_win);
