@@ -53,6 +53,7 @@
 #include "update.h"
 #include "style.h"
 #include "focus.h"
+#include "stack.h"
 #ifdef HAVE_STROKE
 #include "stroke.h"
 #endif /* HAVE_STROKE */
@@ -254,6 +255,10 @@ void WindowShade(F_CMD_ARGS)
       {
 	XMoveWindow(dpy, tmp_win->w, 0, 0);
       }
+      if (Scr.Focus)
+      {
+	focus_grab_buttons(Scr.Focus, True);
+      }
     }
     else
     {
@@ -344,6 +349,16 @@ void WindowShade(F_CMD_ARGS)
     XMoveWindow(dpy, tmp_win->decor_w, 0, 0);
     XRaiseWindow(dpy, tmp_win->decor_w);
     XMapWindow(dpy, tmp_win->Parent);
+
+    if (HAS_BOTTOM_TITLE(tmp_win))
+    {
+      XMoveWindow(dpy, tmp_win->w, 0, 1 - ch);
+    }
+    else
+    {
+      XMoveWindow(dpy, tmp_win->w, 0, 0);
+    }
+
     /* domivogt (28-Dec-1999): For some reason the XMoveResize() on the frame
      * window removes the input focus from the client window.  I have no idea
      * why, but if we explicitly restore the focus here everything works fine.
@@ -607,10 +622,10 @@ void exec_setup(F_CMD_ARGS)
   else /* no arg, so use $SHELL -- not working??? */
   {
     if (getenv("SHELL"))
-      exec_shell_name = strdup(getenv("SHELL"));
+      exec_shell_name = safestrdup(getenv("SHELL"));
     else
       /* if $SHELL not set, use default */
-      exec_shell_name = strdup("/bin/sh");
+      exec_shell_name = safestrdup("/bin/sh");
   }
 }
 
@@ -632,7 +647,7 @@ void exec_function(F_CMD_ARGS)
   else
 #endif
   {
-    cmd = strdup(action);
+    cmd = safestrdup(action);
   }
   if (!cmd)
     return;
@@ -728,7 +743,7 @@ void wait_func(F_CMD_ARGS)
       temp = action;
       while (*temp && isspace((unsigned char)*temp))
         temp++;
-      wait_string = strdup(temp);
+      wait_string = safestrdup(temp);
       for (i = strlen(wait_string) - 1; i >= 0 && isspace(wait_string[i]); i--)
       {
         wait_string[i] = 0;
@@ -737,17 +752,17 @@ void wait_func(F_CMD_ARGS)
   }
   else
   {
-    wait_string = strdup("");
+    wait_string = safestrdup("");
   }
 
-  while(!done && !isTerminated)
+  while (!done && !isTerminated)
   {
     if (BUSY_WAIT & Scr.BusyCursor)
     {
       XDefineCursor(dpy, Scr.Root, Scr.FvwmCursors[CRS_WAIT]);
       redefine_cursor = True;
     }
-    if(My_XNextEvent(dpy, &Event))
+    if (My_XNextEvent(dpy, &Event))
     {
       DispatchEvent(False);
       if(Event.type == MapNotify)
@@ -1180,9 +1195,9 @@ void SetDefaultColors(F_CMD_ARGS)
   if (action)
     action = GetNextToken(action, &back);
   if (!back)
-    back = strdup("grey");
+    back = safestrdup("grey");
   if (!fore)
-    fore = strdup("black");
+    fore = safestrdup("black");
 
   if (!StrEquals(fore, "-"))
   {
@@ -1863,10 +1878,10 @@ void ChangeDecor(F_CMD_ARGS)
     XBell(dpy, 0);
     return;
   }
-  apply_decor_change(tmp_win);
-  old_height = tmp_win->decor->title_height;
+  old_height = (tmp_win->decor) ? tmp_win->decor->title_height : 0;
   tmp_win->decor = found;
-  extra_height = (HAS_TITLE(tmp_win)) ?
+  apply_decor_change(tmp_win);
+  extra_height = (HAS_TITLE(tmp_win) && old_height) ?
     (old_height - tmp_win->decor->title_height) : 0;
   ForceSetupFrame(
     tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y, tmp_win->frame_g.width,
@@ -1916,6 +1931,10 @@ void DestroyDecor(F_CMD_ARGS)
     {
       if (fw->decor == found)
       {
+	/* remove the extra title height now because we delete the current decor
+	 * before calling ChangeDecor(). */
+	fw->frame_g.height -= fw->decor->title_height;
+	fw->decor = NULL;
 	old_execute_function(
 	  "ChangeDecor Default", fw, eventp, C_WINDOW, *Module, 0, NULL);
       }
@@ -2438,9 +2457,9 @@ static void do_recapture(F_CMD_ARGS, Bool fSingle)
   }
   MyXGrabServer(dpy);
   if (fSingle)
-    CaptureOneWindow(tmp_win, tmp_win->w, None, None);
+    CaptureOneWindow(tmp_win, tmp_win->w, None, None, True);
   else
-    CaptureAllWindows();
+    CaptureAllWindows(True);
   /* Throw away queued up events. We don't want user input during a
    * recapture. The window the user clicks in might disapper at the very same
    * moment and the click goes through to the root window. Not good */
@@ -2448,6 +2467,9 @@ static void do_recapture(F_CMD_ARGS, Bool fSingle)
   discard_events(
     ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|PointerMotionMask|
     EnterWindowMask|LeaveWindowMask|KeyPressMask|KeyReleaseMask);
+#ifdef DEBUG_STACK_RING
+  verify_stack_ring_consistency();
+#endif
   MyXUngrabServer(dpy);
 }
 
@@ -2711,6 +2733,22 @@ void SetBugOptions(F_CMD_ARGS)
 	break;
       }
     }
+    else if (StrEquals(opt, "FlickeringQtDialogsWorkaround"))
+    {
+      switch (toggle)
+      {
+      case -1:
+	Scr.bo.FlickeringQtDialogsWorkaround ^= 1;
+	break;
+      case 0:
+      case 1:
+	Scr.bo.FlickeringQtDialogsWorkaround = toggle;
+	break;
+      default:
+	Scr.bo.FlickeringQtDialogsWorkaround = 0;
+	break;
+      }
+    }
     else
     {
       fvwm_msg(ERR, "SetBugOptions", "Unknown Bug Option '%s'", opt);
