@@ -155,6 +155,7 @@ void HandleFocusIn(void)
   Pixel bc = 0;
   FvwmWindow *ffw_old = Scr.Focus;
   Bool do_force_broadcast = False;
+  Bool is_unmanaged_focused = False;
   static Window last_focus_w = None;
   static Window last_focus_fw = None;
   static Bool is_never_focused = True;
@@ -165,7 +166,7 @@ void HandleFocusIn(void)
   if (Event.xfocus.detail != NotifyPointer)
   /**/
     w= Event.xany.window;
-  while(XCheckTypedEvent(dpy,FocusIn,&d))
+  while (XCheckTypedEvent(dpy,FocusIn,&d))
   {
     /* dito */
     if (d.xfocus.detail != NotifyPointer)
@@ -178,7 +179,7 @@ void HandleFocusIn(void)
     return;
   }
   /**/
-  if (XFindContext (dpy, w, FvwmContext, (caddr_t *) &Tmp_win) == XCNOENT)
+  if (XFindContext(dpy, w, FvwmContext, (caddr_t *) &Tmp_win) == XCNOENT)
   {
     Tmp_win = NULL;
   }
@@ -189,6 +190,7 @@ void HandleFocusIn(void)
     {
       Scr.UnknownWinFocused = w;
       focus_w = w;
+      is_unmanaged_focused = True;
     }
     else
     {
@@ -221,7 +223,10 @@ void HandleFocusIn(void)
   {
     do_force_broadcast = IS_FOCUS_CHANGE_BROADCAST_PENDING(Tmp_win);
     SET_FOCUS_CHANGE_BROADCAST_PENDING(Tmp_win, 0);
-    DrawDecorations(Tmp_win, DRAW_ALL, True, True, None);
+    if (Tmp_win != Scr.Hilite)
+    {
+      DrawDecorations(Tmp_win, DRAW_ALL, True, True, None);
+    }
     focus_w = Tmp_win->w;
     focus_fw = Tmp_win->frame;
     fc = Tmp_win->hicolors.fore;
@@ -247,8 +252,11 @@ void HandleFocusIn(void)
       focus_w != last_focus_w || focus_fw != last_focus_fw ||
       do_force_broadcast)
   {
-    BroadcastPacket(M_FOCUS_CHANGE, 5, focus_w, focus_fw,
-                    (unsigned long)IsLastFocusSetByMouse(), fc, bc);
+    if (!Scr.bo.FlickeringQtDialogsWorkaround || !is_unmanaged_focused)
+    {
+      BroadcastPacket(M_FOCUS_CHANGE, 5, focus_w, focus_fw,
+		      (unsigned long)IsLastFocusSetByMouse(), fc, bc);
+    }
     last_focus_w = focus_w;
     last_focus_fw = focus_fw;
     is_never_focused = False;
@@ -866,25 +874,38 @@ void HandleClientMessage(void)
  ***********************************************************************/
 void HandleExpose(void)
 {
+  XRectangle r;
+
   if (Event.xexpose.count != 0)
-    return;
-
-  DBUG("HandleExpose","Routine Entered");
-
+  {
+    flush_accumulate_expose(Event.xexpose.window, &Event);
+  }
+  r.x = Event.xexpose.x;
+  r.y = Event.xexpose.y;
+  r.width = Event.xexpose.width;
+  r.height = Event.xexpose.height;
   if (Tmp_win)
   {
     draw_window_parts draw_parts;
 
     if (Event.xany.window == Tmp_win->title_w)
+    {
       draw_parts = DRAW_TITLE;
+    }
     else if (Event.xany.window == Tmp_win->decor_w ||
 	     Event.xany.window == Tmp_win->frame)
+    {
       draw_parts = DRAW_FRAME;
+    }
     else
+    {
       draw_parts = DRAW_BUTTONS;
-    DrawDecorations(
-      Tmp_win, draw_parts, (Scr.Hilite == Tmp_win), True, Event.xany.window);
+    }
+    draw_clipped_decorations(
+      Tmp_win, draw_parts, (Scr.Hilite == Tmp_win), True, Event.xany.window,
+      &r);
   }
+
   return;
 }
 
@@ -907,6 +928,48 @@ void HandleDestroyNotify(void)
 
 
 
+/***********************************************************************
+ *
+ *  Procedure:
+ *	SendConfigureNotify - inform a client window of its geometry.
+ *
+ *  The input (frame) geometry will be translated to client geometry
+ *  before sending.
+ *
+ ************************************************************************/
+static void fake_map_unmap_notify(FvwmWindow *tmp_win, int event_type)
+{
+  XEvent client_event;
+  XWindowAttributes winattrs = {0};
+
+  if (!XGetWindowAttributes(dpy, tmp_win->w, &winattrs))
+  {
+    return;
+  }
+  XSelectInput(
+    dpy, tmp_win->w, winattrs.your_event_mask & ~StructureNotifyMask);
+  client_event.type = event_type;
+  client_event.xmap.display = dpy;
+  client_event.xmap.event = tmp_win->w;
+  client_event.xmap.window = tmp_win->w;
+  switch (event_type)
+  {
+    case MapNotify:
+      client_event.xmap.override_redirect = False;
+      break;
+    case UnmapNotify:
+      client_event.xunmap.from_configure = False;
+      break;
+    default:
+      /* not possible if called correctly */
+      break;
+  }
+  XSendEvent(dpy, tmp_win->w, False, StructureNotifyMask, &client_event);
+  XSelectInput(dpy, tmp_win->w, winattrs.your_event_mask);
+
+  return;
+}
+
 /***********************************************************************
  *
  *  Procedure:
@@ -931,7 +994,9 @@ void HandleMapRequestKeepRaised(Window KeepRaised, FvwmWindow *ReuseWin)
   extern long isIconicState;
   extern Bool isIconifiedByParent;
   extern Bool PPosOverride;
-  Bool OnThisPage = False;
+  Bool is_on_this_page = False;
+  Bool is_new_window = False;
+  FvwmWindow *tmp;
 
   Event.xany.window = Event.xmaprequest.window;
 
@@ -948,30 +1013,42 @@ void HandleMapRequestKeepRaised(Window KeepRaised, FvwmWindow *ReuseWin)
     Tmp_win = ReuseWin;
   }
 
-  if(!PPosOverride)
+  if (!PPosOverride)
+  {
     XFlush(dpy);
+  }
 
   /* If the window has never been mapped before ... */
-  if(!Tmp_win || (Tmp_win && DO_REUSE_DESTROYED(Tmp_win)))
+  if (!Tmp_win || (Tmp_win && DO_REUSE_DESTROYED(Tmp_win)))
   {
     /* Add decorations. */
     Tmp_win = AddWindow(Event.xany.window, ReuseWin);
     if (Tmp_win == NULL)
       return;
+    is_new_window = True;
   }
   /*
    * Make sure at least part of window is on this page
    * before giving it focus...
    */
-  OnThisPage = IsRectangleOnThisPage(&(Tmp_win->frame_g), Tmp_win->Desk);
-
+  is_on_this_page = IsRectangleOnThisPage(&(Tmp_win->frame_g), Tmp_win->Desk);
   if(KeepRaised != None)
   {
     XRaiseWindow(dpy, KeepRaised);
   }
   /* If it's not merely iconified, and we have hints, use them. */
 
-  if (!IS_ICONIFIED(Tmp_win))
+  if (IS_ICONIFIED(Tmp_win))
+  {
+    /* If no hints, or currently an icon, just "deiconify" */
+    DeIconify(Tmp_win);
+  }
+  else if (IS_MAPPED(Tmp_win))
+  {
+    /* the window is already mapped - fake a MapNotify event */
+    fake_map_unmap_notify(Tmp_win, MapNotify);
+  }
+  else
   {
     int state;
 
@@ -1003,7 +1080,7 @@ void HandleMapRequestKeepRaised(Window KeepRaised, FvwmWindow *ReuseWin)
 	SetMapStateProp(Tmp_win, NormalState);
 	if (Scr.flags.is_map_desk_in_progress)
 	  do_grab_focus = False;
-	else if (!OnThisPage)
+	else if (!is_on_this_page)
 	  do_grab_focus = False;
 	else if (DO_GRAB_FOCUS(Tmp_win) &&
 		 (!IS_TRANSIENT(Tmp_win) || Tmp_win->transientfor == Scr.Root))
@@ -1061,12 +1138,14 @@ void HandleMapRequestKeepRaised(Window KeepRaised, FvwmWindow *ReuseWin)
       break;
 
     case IconicState:
-      if (isIconifiedByParent)
+      if (isIconifiedByParent ||
+	  ((tmp = get_transientfor_fvwmwindow(Tmp_win)) && IS_ICONIFIED(tmp)))
       {
 	isIconifiedByParent = False;
 	SET_ICONIFIED_BY_PARENT(Tmp_win, 1);
       }
-      if (USE_ICON_POSITION_HINT(Tmp_win) && Tmp_win->wmhints)
+      if (USE_ICON_POSITION_HINT(Tmp_win) && Tmp_win->wmhints &&
+	  (Tmp_win->wmhints->flags & IconPositionHint))
       {
 	Iconify(Tmp_win, Tmp_win->wmhints->icon_x,
 		Tmp_win->wmhints->icon_y);
@@ -1075,6 +1154,11 @@ void HandleMapRequestKeepRaised(Window KeepRaised, FvwmWindow *ReuseWin)
       {
 	Iconify(Tmp_win, 0, 0);
       }
+      if (is_new_window)
+      {
+	/* the window will not be mapped - fake an UnmapNotify event */
+	fake_map_unmap_notify(Tmp_win, UnmapNotify);
+      }
       break;
     }
 #if 0
@@ -1084,11 +1168,6 @@ void HandleMapRequestKeepRaised(Window KeepRaised, FvwmWindow *ReuseWin)
 #endif
     MyXUngrabServer(dpy);
   }
-  /* If no hints, or currently an icon, just "deiconify" */
-  else
-  {
-    DeIconify(Tmp_win);
-  }
   if (IS_SHADED(Tmp_win))
   {
     BroadcastPacket(M_WINDOWSHADE, 3, Tmp_win->w, Tmp_win->frame,
@@ -1136,7 +1215,7 @@ void HandleMapRequestKeepRaised(Window KeepRaised, FvwmWindow *ReuseWin)
  ***********************************************************************/
 void HandleMapNotify(void)
 {
-  Bool OnThisPage = False;
+  Bool is_on_this_page = False;
 
   DBUG("HandleMapNotify","Routine Entered");
 
@@ -1167,7 +1246,7 @@ void HandleMapNotify(void)
       Make sure at least part of window is on this page
       before giving it focus...
   */
-  OnThisPage = IsRectangleOnThisPage(&(Tmp_win->frame_g), Tmp_win->Desk);
+  is_on_this_page = IsRectangleOnThisPage(&(Tmp_win->frame_g), Tmp_win->Desk);
 
   /*
    * Need to do the grab to avoid race condition of having server send
@@ -1182,12 +1261,10 @@ void HandleMapNotify(void)
     XUnmapWindow(dpy, Tmp_win->icon_pixmap_w);
   XMapSubwindows(dpy, Tmp_win->frame);
   XMapSubwindows(dpy, Tmp_win->decor_w);
-
-  if(Tmp_win->Desk == Scr.CurrentDesk)
+  if (Tmp_win->Desk == Scr.CurrentDesk)
   {
     XMapWindow(dpy, Tmp_win->frame);
   }
-
   if(IS_ICONIFIED(Tmp_win))
     BroadcastPacket(M_DEICONIFY, 3,
                     Tmp_win->w, Tmp_win->frame, (unsigned long)Tmp_win);
@@ -1199,7 +1276,7 @@ void HandleMapNotify(void)
      (IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS_TRANSIENT(Tmp_win) &&
       Scr.Focus && Scr.Focus->w == Tmp_win->transientfor))
   {
-    if (OnThisPage)
+    if (is_on_this_page)
     {
       SetFocusWindow(Tmp_win, 1);
     }
@@ -1272,18 +1349,14 @@ void HandleUnmapNotify(void)
 		     FvwmContext, (caddr_t *)&Tmp_win) == XCNOENT)
       Tmp_win = NULL;
   }
-
-  if(!Tmp_win)
+  if (!Tmp_win)
     return;
-
   if (Event.xunmap.window == Tmp_win->frame)
   {
     SET_DEICONIFY_PENDING(Tmp_win , 0);
   }
-
   if (must_return)
     return;
-
   if(weMustUnmap)
     XUnmapWindow(dpy, Event.xunmap.window);
   if(Tmp_win ==  Scr.Hilite)
@@ -1301,13 +1374,6 @@ void HandleUnmapNotify(void)
     return;
   }
 
-  MyXGrabServer(dpy);
-
-  if(XCheckTypedWindowEvent (dpy, Event.xunmap.window, DestroyNotify,&dummy))
-  {
-    destroy_window(Tmp_win);
-  }
-  else
   /*
    * The program may have unmapped the client window, from either
    * NormalState or IconicState.  Handle the transition to WithdrawnState.
@@ -1316,12 +1382,14 @@ void HandleUnmapNotify(void)
    * won't cause it to get mapped) and then throw away all state (pretend
    * that we've received a DestroyNotify).
    */
-  if (XTranslateCoordinates (dpy, Event.xunmap.window, Scr.Root,
+  if (!XCheckTypedWindowEvent (dpy, Event.xunmap.window, DestroyNotify, &dummy)
+   && XTranslateCoordinates (dpy, Event.xunmap.window, Scr.Root,
 			     0, 0, &dstx, &dsty, &dumwin))
   {
     XEvent ev;
     Bool reparented;
 
+    MyXGrabServer(dpy);
     reparented = XCheckTypedWindowEvent (dpy, Event.xunmap.window,
 					 ReparentNotify, &ev);
     SetMapStateProp (Tmp_win, WithdrawnState);
@@ -1339,19 +1407,9 @@ void HandleUnmapNotify(void)
     }
     XRemoveFromSaveSet (dpy, Event.xunmap.window);
     XSelectInput (dpy, Event.xunmap.window, NoEventMask);
-    destroy_window(Tmp_win);		/* do not need to mash event before */
-    /*
-     * Flush any pending events for the window.
-     */
-    /* Bzzt! it could be about to re-map */
-/*      while(XCheckWindowEvent(dpy, Event.xunmap.window,
-			      StructureNotifyMask | PropertyChangeMask |
-			      ColormapChangeMask | VisibilityChangeMask |
-			      EnterWindowMask | LeaveWindowMask, &dummy));
-*/
-  } /* else window no longer exists and we'll get a destroy notify */
-  MyXUngrabServer(dpy);
-
+    MyXUngrabServer(dpy);
+  }
+  destroy_window(Tmp_win);		/* do not need to mash event before */
   if (focus_grabbed)
   {
     CoerceEnterNotifyOnCurrentWindow();
@@ -2070,6 +2128,17 @@ fprintf(stderr, "cre: %d(%d) %d(%d) %d(%d)x%d(%d) w 0x%08x '%s'\n",
     dw = 0;
     dh = 0;
 
+    if (IS_SHADED(Tmp_win))
+    {
+      /* forbid shaded applications to move their windows */
+      cre->value_mask &= ~(CWX | CWY);
+    }
+    if (IS_MAXIMIZED(Tmp_win))
+    {
+      /* dont allow clients to resize maximized windows */
+      cre->value_mask &= ~(CWWidth | CWHeight);
+    }
+
     /* for restoring */
     if (cre->value_mask & CWBorderWidth)
     {
@@ -2150,18 +2219,17 @@ fprintf(stderr, "cre: %d(%d) %d(%d) %d(%d)x%d(%d) w 0x%08x '%s'\n",
       gravity_resize(Tmp_win->hints.win_gravity, &new_g, 0, dh);
     }
 
-    /* dont allow clients to resize maximized windows */
-    if (!IS_MAXIMIZED(Tmp_win) || (!dw && !dh))
+    if (dx || dy || dw || dh)
     {
       if (IS_SHADED(Tmp_win))
 	get_shaded_geometry(Tmp_win, &new_g, &new_g);
       SetupFrame(
 	Tmp_win, new_g.x, new_g.y, new_g.width, new_g.height, False);
+      /* make sure the window structure has the new position */
+      update_absolute_geometry(Tmp_win);
+      maximize_adjust_offset(Tmp_win);
+      GNOME_SetWinArea(Tmp_win);
     }
-    /* make sure the window structure has the new position */
-    update_absolute_geometry(Tmp_win);
-    maximize_adjust_offset(Tmp_win);
-    GNOME_SetWinArea(Tmp_win);
   }
 
   /*  Stacking order change requested...  */
@@ -2288,36 +2356,36 @@ void SendConfigureNotify(
   FvwmWindow *tmp_win, int x, int y, unsigned int w, unsigned int h, int bw,
   Bool send_for_frame_too)
 {
+  XEvent client_event;
+
   if (!tmp_win || IS_SHADED(tmp_win))
     return;
+  client_event.type = ConfigureNotify;
+  client_event.xconfigure.display = dpy;
+  client_event.xconfigure.event = tmp_win->w;
+  client_event.xconfigure.window = tmp_win->w;
+  client_event.xconfigure.x = x + tmp_win->boundary_width;
+  client_event.xconfigure.y = y + tmp_win->boundary_width +
+    tmp_win->title_top_height;
+  client_event.xconfigure.width = w - 2 * tmp_win->boundary_width;
+  client_event.xconfigure.height = h -
+    2 * tmp_win->boundary_width - tmp_win->title_g.height;
+  client_event.xconfigure.border_width = bw;
+  client_event.xconfigure.above = tmp_win->frame;
+  client_event.xconfigure.override_redirect = False;
+  XSendEvent(dpy, tmp_win->w, False, StructureNotifyMask, &client_event);
+  if (send_for_frame_too)
   {
-    XEvent client_event;
-
-    client_event.type = ConfigureNotify;
-    client_event.xconfigure.display = dpy;
-    client_event.xconfigure.event = tmp_win->w;
-    client_event.xconfigure.window = tmp_win->w;
-    client_event.xconfigure.x = x + tmp_win->boundary_width;
-    client_event.xconfigure.y = y + tmp_win->boundary_width +
-      tmp_win->title_top_height;
-    client_event.xconfigure.width = w - 2 * tmp_win->boundary_width;
-    client_event.xconfigure.height = h -
-      2 * tmp_win->boundary_width - tmp_win->title_g.height;
-    client_event.xconfigure.border_width = bw;
-    client_event.xconfigure.above = tmp_win->frame;
-    client_event.xconfigure.override_redirect = False;
-    XSendEvent(dpy, tmp_win->w, False, StructureNotifyMask, &client_event);
-    if (send_for_frame_too)
-    {
-      /* This is for buggy tk, which waits for the real ConfigureNotify
-       * on frame instead of the synthetic one on w. The geometry data
-       * in the event will not be correct for the frame, but tk doesn't
-       * look at that data anyway. */
-      client_event.xconfigure.event = tmp_win->frame;
-      client_event.xconfigure.window = tmp_win->frame;
-      XSendEvent(dpy, tmp_win->frame, False,StructureNotifyMask,&client_event);
-    }
+    /* This is for buggy tk, which waits for the real ConfigureNotify
+     * on frame instead of the synthetic one on w. The geometry data
+     * in the event will not be correct for the frame, but tk doesn't
+     * look at that data anyway. */
+    client_event.xconfigure.event = tmp_win->frame;
+    client_event.xconfigure.window = tmp_win->frame;
+    XSendEvent(dpy, tmp_win->frame, False,StructureNotifyMask,&client_event);
   }
+
+  return;
 }
 
 /***********************************************************************
@@ -2798,7 +2866,7 @@ int flush_expose (Window w)
   XEvent dummy;
   int i=0;
 
-  while (XCheckTypedWindowEvent (dpy, w, Expose, &dummy))
+  while (XCheckTypedWindowEvent(dpy, w, Expose, &dummy))
     i++;
   return i;
 }
