@@ -206,6 +206,7 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
   {
     FOCUS_SET(w);
     Scr.Focus = Fw;
+    SET_FOCUS_CHANGE_BROADCAST_PENDING(Fw, 1);
     Scr.UnknownWinFocused = None;
   }
   else if (!Fw || !(Fw->wmhints) || !(Fw->wmhints->flags & InputHint) ||
@@ -214,6 +215,10 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
     /* Window will accept input focus */
     FOCUS_SET(w);
     Scr.Focus = Fw;
+    if (Fw)
+    {
+      SET_FOCUS_CHANGE_BROADCAST_PENDING(Fw, 1);
+    }
     Scr.UnknownWinFocused = None;
   }
   else if ((Scr.Focus)&&(Scr.Focus->Desk == Scr.CurrentDesk))
@@ -556,7 +561,6 @@ void focus_grab_buttons(FvwmWindow *tmp_win, Bool is_focused)
     return;
   }
   accepts_input_focus = do_accept_input_focus(tmp_win);
-
   if ((HAS_SLOPPY_FOCUS(tmp_win) || HAS_MOUSE_FOCUS(tmp_win) ||
        HAS_NEVER_FOCUS(tmp_win)) &&
       DO_RAISE_MOUSE_FOCUS_CLICK(tmp_win) &&
