@@ -135,6 +135,7 @@ Window JunkRoot, JunkChild;		/* junk window */
 unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
 
 Bool debugging = False;
+Bool debugging_stack_ring = False;
 Bool PPosOverride = False;
 
 Window bad_window = None;
@@ -227,7 +228,7 @@ int main(int argc, char **argv)
   if (home_dir == NULL) {
     struct passwd* pw = getpwuid(getuid());
     if (pw != NULL)
-      home_dir = strdup(pw->pw_dir);
+      home_dir = safestrdup(pw->pw_dir);
   }
 #endif
   if (home_dir == NULL)
@@ -237,10 +238,10 @@ int main(int argc, char **argv)
   fvwm_userdir = getenv("FVWM_USERDIR");
   if (fvwm_userdir == NULL)
   {
-    fvwm_userdir = strdup(CatString2(home_dir, "/.fvwm"));
+    fvwm_userdir = safestrdup(CatString2(home_dir, "/.fvwm"));
     /* Put the user directory into the environment so it can be used
        later everywhere. */
-    putenv(strdup(CatString2("FVWM_USERDIR=", fvwm_userdir)));
+    putenv(safestrdup(CatString2("FVWM_USERDIR=", fvwm_userdir)));
   }
 
   /* Create FVWM_USERDIR directory if needed */
@@ -251,22 +252,26 @@ int main(int argc, char **argv)
 
   for (i = 1; i < argc; i++)
   {
-    if (strncasecmp(argv[i],"-debug",6)==0)
+    if (strncasecmp(argv[i],"-debug_stack_ring", 17)==0)
+    {
+      debugging_stack_ring = True;
+    }
+    else if (strncasecmp(argv[i],"-debug",6)==0)
     {
       debugging = True;
     }
     else if (strncasecmp(argv[i], "-clientId", 9) == 0)
-      {
-	if (++i >= argc)
-	  usage();
-	SetClientID(argv[i]);
-      }
+    {
+      if (++i >= argc)
+	usage();
+      SetClientID(argv[i]);
+    }
     else if (strncasecmp(argv[i], "-restore", 8) == 0)
-      {
-	if (++i >= argc)
-	  usage();
-	state_filename = argv[i];
-      }
+    {
+      if (++i >= argc)
+	usage();
+      state_filename = argv[i];
+    }
     else if (strncasecmp(argv[i],"-s",2)==0)
     {
       do_force_single_screen = True;
@@ -300,7 +305,7 @@ int main(int argc, char **argv)
         usage();
       if (num_config_commands < MAX_CFG_CMDS)
       {
-        config_commands[num_config_commands] = strdup(argv[i]);
+        config_commands[num_config_commands] = safestrdup(argv[i]);
         num_config_commands++;
       }
       else
@@ -421,7 +426,7 @@ int main(int argc, char **argv)
     g_argv[argc++] = "-s";
 /*
     g_argv[argc++] = "-d";
-    g_argv[argc++] = strdup(XDisplayString(dpy));
+    g_argv[argc++] = safestrdup(XDisplayString(dpy));
 */
     g_argv[argc] = NULL;
   }
@@ -618,7 +623,7 @@ int main(int argc, char **argv)
   }
 
   restart_state_filename =
-    strdup(CatString3(fvwm_userdir, "/.fs-restart-", getenv("HOSTDISPLAY")));
+    safestrdup(CatString3(fvwm_userdir, "/.fs-restart-", getenv("HOSTDISPLAY")));
   if (!state_filename && Restarting)
     state_filename = restart_state_filename;
 
@@ -786,7 +791,7 @@ void StartupStuff(void)
 #define startFuncName "StartFunction"
   const char *initFuncName;
 
-  CaptureAllWindows();
+  CaptureAllWindows(False);
   /* Turn off the SM stuff after the initial capture so that new windows will
    * not be matched by accident. */
   if (Restarting)
@@ -815,7 +820,7 @@ void StartupStuff(void)
   /* migo (03-Jul-1999): execute [Session]{Init|Restart}Function */
   initFuncName = getInitFunctionName(Restarting == True);
   if (FindFunction(initFuncName)) {
-    char *action = strdup(CatString2("Function ", initFuncName));
+    char *action = safestrdup(CatString2("Function ", initFuncName));
     old_execute_function(action, NULL, &Event, C_ROOT, 1, 0, NULL);
     free(action);
   }
@@ -847,7 +852,8 @@ void StartupStuff(void)
  ***********************************************************************/
 
 void CaptureOneWindow(
-  FvwmWindow *fw, Window window, Window keep_on_top_win, Window parent_win)
+  FvwmWindow *fw, Window window, Window keep_on_top_win, Window parent_win,
+  Bool is_recapture)
 {
   Window w;
   unsigned long data[1];
@@ -905,7 +911,7 @@ void CaptureOneWindow(
     XSelectInput(dpy, fw->w, 0);
     w = fw->w;
     XUnmapWindow(dpy, fw->frame);
-    RestoreWithdrawnLocation(fw, True, parent_win);
+    RestoreWithdrawnLocation(fw, is_recapture, parent_win);
     SET_DO_REUSE_DESTROYED(fw, 1); /* RBW - 1999/03/20 */
     destroy_window(fw);
     Event.xmaprequest.window = w;
@@ -991,7 +997,7 @@ static void hide_screen(
   return;
 }
 
-void CaptureAllWindows(void)
+void CaptureAllWindows(Bool is_recapture)
 {
   int i,j;
   unsigned int nchildren;
@@ -1043,7 +1049,7 @@ void CaptureAllWindows(void)
       {
         XUnmapWindow(dpy, children[i]);
         Event.xmaprequest.window = children[i];
-        HandleMapRequestKeepRaised (None, NULL);
+        HandleMapRequestKeepRaised(None, NULL);
       }
     }
     Scr.flags.windows_captured = 1;
@@ -1062,7 +1068,8 @@ void CaptureAllWindows(void)
     {
       if (XFindContext(dpy, children[i], FvwmContext, (caddr_t *)&tmp)!=XCNOENT)
       {
-        CaptureOneWindow(tmp, children[i], keep_on_top_win, parent_win);
+        CaptureOneWindow(
+	  tmp, children[i], keep_on_top_win, parent_win, is_recapture);
       }
     }
     hide_screen(False, NULL, NULL);
@@ -1816,8 +1823,8 @@ static void InitVariables(void)
 #else
   Scr.bo.DisableConfigureNotify = 0;
 #endif
-
   Scr.bo.InstallRootCmap = 0;
+  Scr.bo.FlickeringQtDialogsWorkaround = 1;
 
   Scr.gs.EmulateMWM = DEFAULT_EMULATE_MWM;
   Scr.gs.EmulateWIN = DEFAULT_EMULATE_WIN;
@@ -1903,7 +1910,7 @@ void Done(int restart, char *command)
   exitFuncName = getInitFunctionName(2);
   if (FindFunction(exitFuncName))
   {
-    char *action = strdup(CatString2("Function ", exitFuncName));
+    char *action = safestrdup(CatString2("Function ", exitFuncName));
     old_execute_function(action, NULL, &Event, C_ROOT, 1, 0, NULL);
     free(action);
   }
@@ -2287,7 +2294,7 @@ static void setVersionInfo(void)
   /* Set version information string */
   sprintf(version_str, "FVWM version %s compiled on %s at %s",
           VERSION, __DATE__, __TIME__);
-  Fvwm_VersionInfo = strdup(version_str);
+  Fvwm_VersionInfo = safestrdup(version_str);
 
 #ifdef HAVE_READLINE
   strcat(support_str, " ReadLine,");
@@ -2313,15 +2320,15 @@ static void setVersionInfo(void)
   strcat(support_str, " SM,");
 #endif
 #ifdef I18N_MB
-  strcat(support_str, " I18N_MB,");
+  strcat(support_str, " Multibyte,");
 #endif
 
   support_len = strlen(support_str);
-  if (support_len)
+  if (support_len > 0)
   {
     /* strip last comma */
     support_str[support_len - 1] = '\0';
-    Fvwm_SupportInfo = strdup(CatString2("with support for:", support_str));
+    Fvwm_SupportInfo = safestrdup(CatString2("with support for:", support_str));
   }
   else
     Fvwm_SupportInfo = "with no optional feature support";
