@@ -199,8 +199,11 @@ typedef struct
     unsigned do_circulate_skip_shaded : 1;
     unsigned do_grab_focus_when_created : 1;
     unsigned do_grab_focus_when_transient_created : 1;
+    unsigned do_iconify_window_groups : 1;
     unsigned do_ignore_gnome_hints : 1;
+    unsigned do_ignore_icon_boxes : 1;
     unsigned do_ignore_restack : 1;
+    unsigned do_use_window_group_hint : 1;
     unsigned do_lower_transient : 1;
     unsigned do_not_show_on_map : 1;
     unsigned do_not_pass_click_focus_click : 1;
@@ -289,6 +292,8 @@ typedef struct
   unsigned is_window_drawn_once : 1;
   unsigned is_viewport_moved : 1; /* To prevent double move in MoveViewport.*/
   unsigned is_window_being_moved_opaque : 1;
+  unsigned is_window_border_drawn : 1; /* 0 at first, set to 1 after border has
+					* been drawn once */
   unsigned is_window_font_loaded : 1;
   unsigned is_window_shaded : 1;
   unsigned using_default_icon_font : 1;
