@@ -719,6 +719,31 @@ void AutoPlaceIcon(FvwmWindow *t)
      Look thru chain of icon boxes assigned to window.
      Add logic for grids and fill direction.
   */
+  else if (DO_IGNORE_ICON_BOXES(t))
+  {
+    t->icon_g.x = t->frame_g.x;
+    if (HAS_BOTTOM_TITLE(t))
+    {
+      t->icon_g.y =
+	t->frame_g.y + t->frame_g.height - t->icon_g.height - t->icon_p_height;
+    }
+    else
+    {
+      t->icon_g.y = t->frame_g.y;
+    }
+    if (t->icon_g.x < 0)
+      t->icon_g.x = 0;
+    else if (t->icon_g.x + t->icon_g.width > Scr.MyDisplayWidth)
+      t->icon_g.x = Scr.MyDisplayWidth - t->icon_g.width;
+    if (t->icon_g.y < 0)
+      t->icon_g.y = 0;
+    else if (t->icon_g.y + t->icon_g.height + t->icon_p_height >
+	     Scr.MyDisplayHeight)
+      t->icon_g.y =
+	Scr.MyDisplayHeight - t->icon_g.height - t->icon_p_height;
+    do_move_pixmap = True;
+    do_move_title = True;
+  }
   else
   {
     /* A place to hold inner and outer loop variables. */
@@ -1207,12 +1232,14 @@ void DeIconify(FvwmWindow *tmp_win)
   }
 
   /* AS dje  RaiseWindow(tmp_win); */
+
+  mark_transient_subtree(tmp_win, MARK_ALL_LAYERS, MARK_ALL, False, True);
   /* now de-iconify transients */
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
   {
-    if ((t == tmp_win)||
-	((IS_TRANSIENT(t)) && (t->transientfor == tmp_win->w)))
+    if (t == tmp_win || IS_IN_TRANSIENT_SUBTREE(t))
     {
+      SET_IN_TRANSIENT_SUBTREE(t, 0);
       SET_MAPPED(t, 1);
       SET_ICONIFIED_BY_PARENT(t, 0);
       if(Scr.Hilite == t)
@@ -1276,7 +1303,7 @@ void DeIconify(FvwmWindow *tmp_win)
 			(unsigned long)t,
 			t->icon_g.x, t->icon_g.y,
 			t->icon_p_width,
-			t->icon_p_height+t->icon_g.height);
+			t->icon_p_height + t->icon_g.height);
       }
       if(t->Desk == Scr.CurrentDesk)
       {
@@ -1300,7 +1327,10 @@ void DeIconify(FvwmWindow *tmp_win)
 #if 1
   RaiseWindow(tmp_win); /* moved dje */
 #endif
-
+  if (Scr.Focus)
+  {
+    focus_grab_buttons(Scr.Focus, True);
+  }
   if(HAS_CLICK_FOCUS(tmp_win))
     FocusOn(tmp_win, TRUE, "");
   GNOME_SetWinArea(tmp_win);
@@ -1341,11 +1371,13 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
     SetFocusWindow(tmp_win->next, 1);
   }
 
+  mark_transient_subtree(tmp_win, MARK_ALL_LAYERS, MARK_ALL, False, True);
   /* iconify transients first */
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
   {
-    if ((t==tmp_win)||((IS_TRANSIENT(t)) && (t->transientfor == tmp_win->w)))
+    if (t == tmp_win || IS_IN_TRANSIENT_SUBTREE(t))
     {
+      SET_IN_TRANSIENT_SUBTREE(t, 0);
       SET_ICON_ENTERED(t, 0);
       /*
        * Prevent the receipt of an UnmapNotify, since that would
@@ -1364,7 +1396,7 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
 
       SetMapStateProp(t, IconicState);
       DrawDecorations(t, DRAW_ALL, False, False, None);
-      if (t == tmp_win)
+      if (t == tmp_win && !IS_ICONIFIED_BY_PARENT(tmp_win))
       {
 	SET_DEICONIFY_PENDING(t, 1);
       }
@@ -1451,7 +1483,7 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
     LowerWindow(tmp_win);
   }
 
-  if(tmp_win->Desk == Scr.CurrentDesk)
+  if (tmp_win->Desk == Scr.CurrentDesk)
   {
     if (tmp_win->icon_w != None)
       XMapWindow(dpy, tmp_win->icon_w);
@@ -1459,7 +1491,7 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
     if(tmp_win->icon_pixmap_w != None)
       XMapWindow(dpy, tmp_win->icon_pixmap_w);
   }
-  if(HAS_CLICK_FOCUS(tmp_win) || HAS_SLOPPY_FOCUS(tmp_win))
+  if (HAS_CLICK_FOCUS(tmp_win) || HAS_SLOPPY_FOCUS(tmp_win))
   {
     if (tmp_win == Scr.Focus)
     {
@@ -1471,6 +1503,12 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
 	DeleteFocus(1);
     }
   }
+  if (Scr.Focus)
+  {
+    focus_grab_buttons(Scr.Focus, True);
+  }
+
+  return;
 }
 
 
