@@ -354,7 +354,7 @@ static float test_fit( FvwmWindow *t, int x11, int y11, float aoimin,
   float aoi = 0;      /* area of interference */
   float anew;
   float cover_factor = 1;
-  int avoidance_factor;
+  float avoidance_factor;
   int PageBottom    =  Scr.MyDisplayHeight - pdeltay;
   int PageRight     =  Scr.MyDisplayWidth - pdeltax;
   int stickyx, stickyy;
@@ -411,10 +411,12 @@ static float test_fit( FvwmWindow *t, int x11, int y11, float aoimin,
         avoidance_factor = PLACEMENT_AVOID_ICON;
       else if(compare_window_layers(testw, t) > 0)
         avoidance_factor = PLACEMENT_AVOID_ONTOP;
+      else if(compare_window_layers(testw, t) < 0)
+        avoidance_factor = PLACEMENT_AVOID_BELOW;
       else if(IS_STICKY(testw))
         avoidance_factor = PLACEMENT_AVOID_STICKY;
       else
-        avoidance_factor = 1;
+        avoidance_factor = 1.0;
 
       if (use_percent)
       {
@@ -433,7 +435,8 @@ static float test_fit( FvwmWindow *t, int x11, int y11, float aoimin,
 	    cover_factor = PLACEMENT_AVOID_COVER_75;
 	}
 	if (avoidance_factor>1)
-	  avoidance_factor = avoidance_factor + (cover_factor>1)? cover_factor:0;
+	  avoidance_factor =
+	    avoidance_factor + (cover_factor>1) ? cover_factor : 0;
 	else
 	  avoidance_factor = cover_factor;
       }
@@ -485,7 +488,8 @@ Bool PlaceWindow(
    * 2. Windows specified with StartsOnDesk go where specified
    * 3. Put it on the desk it was on before the restart.
    * 4. Transients go on the same desk as their parents.
-   * 5. Window groups stay together (completely untested)
+   * 5. Window groups stay together (if the KeepWindowGroupsOnDesk style is
+   *    used).
    */
 
   /*  RBW - 11/02/1998  */
@@ -554,18 +558,27 @@ Bool PlaceWindow(
     tmp_win->Desk = (Desk > -1) ? Desk - 1 : Desk;    /*  RBW - 11/20/1998  */
   else
   {
-    if((tmp_win->wmhints)&&(tmp_win->wmhints->flags & WindowGroupHint)&&
-       (tmp_win->wmhints->window_group != None)&&
+    if((DO_USE_WINDOW_GROUP_HINT(tmp_win)) &&
+       (tmp_win->wmhints) && (tmp_win->wmhints->flags & WindowGroupHint)&&
+       (tmp_win->wmhints->window_group != None) &&
        (tmp_win->wmhints->window_group != Scr.Root))
     {
-      /* Try to find the group leader or another window
-       * in the group */
+      /* Try to find the group leader or another window in the group */
       for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
       {
-	if((t->w == tmp_win->wmhints->window_group)||
-	   ((t->wmhints)&&(t->wmhints->flags & WindowGroupHint)&&
-	    (t->wmhints->window_group==tmp_win->wmhints->window_group)))
+	if (t->w == tmp_win->wmhints->window_group)
+	{
+	  /* found the group leader, break out */
 	  tmp_win->Desk = t->Desk;
+	  break;
+	}
+	else if (t->wmhints && (t->wmhints->flags & WindowGroupHint) &&
+		 (t->wmhints->window_group == tmp_win->wmhints->window_group))
+	{
+	  /* found a window from the same group, but keep looking for the group
+	   * leader */
+	  tmp_win->Desk = t->Desk;
+	}
       }
     }
     if((IS_TRANSIENT(tmp_win))&&(tmp_win->transientfor!=None)&&
