@@ -31,6 +31,7 @@
 #include "defaults.h"
 #include "module_interface.h"
 #include "stack.h"
+#include "events.h"
 #include "borders.h"
 #include "virtual.h"
 #include "gnome.h"
@@ -52,6 +53,8 @@ static void dump_stack_ring(void)
 {
   FvwmWindow *t1;
 
+  if (!debugging_stack_ring)
+    return;
   XBell(dpy, 0);
   fprintf(stderr,"dumping stack ring:\n");
   for (t1 = Scr.FvwmRoot.stack_next; t1 != &Scr.FvwmRoot; t1 = t1->stack_next)
@@ -64,7 +67,7 @@ static void dump_stack_ring(void)
 }
 
 /* debugging function */
-static void verify_stack_ring_consistency(void)
+void verify_stack_ring_consistency(void)
 {
   Window root, parent, *children;
   unsigned int nchildren, i;
@@ -72,6 +75,8 @@ static void verify_stack_ring_consistency(void)
   int last_layer;
   int last_index;
 
+  if (!debugging_stack_ring)
+    return;
   XSync(dpy, 0);
   t2 = Scr.FvwmRoot.stack_next;
   if (t2 == &Scr.FvwmRoot)
@@ -220,7 +225,7 @@ FvwmWindow *get_transientfor_fvwmwindow(FvwmWindow *t)
   {
     if (s->w == t->transientfor)
     {
-      return s;
+      return (s == t) ? NULL : s;
     }
   }
 
@@ -389,6 +394,10 @@ static Bool must_move_transients(
 {
   *found_transient = False;
 
+  if (IS_ICONIFIED(t))
+  {
+    return False;
+  }
   /* raise */
   if ((!do_lower && DO_RAISE_TRANSIENT(t)) ||
       (do_lower && DO_LOWER_TRANSIENT(t)))
@@ -527,7 +536,7 @@ static void RaiseOrLowerWindow(
    * recursion stuff for new windows because the must_move_transients() call
    * needs a properly ordered stack ring - but the new window is still at the
    * front of the stack ring. */
-  if (allow_recursion && !is_new_window)
+  if (allow_recursion && !is_new_window && !IS_ICONIFIED(t))
   {
     /*
      * This part makes Raise/Lower on a Transient act on its Main and sibling
@@ -547,6 +556,10 @@ static void RaiseOrLowerWindow(
       {
         if (t2->w == t->transientfor)
         {
+	  if (IS_ICONIFIED(t2))
+	  {
+	    break;
+	  }
           if ((!do_lower && DO_RAISE_TRANSIENT(t2)) ||
               (do_lower && DO_LOWER_TRANSIENT(t2))  )
           {
@@ -599,7 +612,7 @@ static void RaiseOrLowerWindow(
       {
 	next = t2->stack_next;
 	if ((IS_TRANSIENT(t2)) && (t2->transientfor == t->w) &&
-	    (t2->layer == t->layer))
+	    (t2->layer == t->layer) && !IS_ICONIFIED(t2))
 	{
 	  /* t2 is a transient to lower */
 	  count++;
@@ -646,7 +659,8 @@ static void RaiseOrLowerWindow(
     */
     add_window_to_stack_ring_after(t, s->stack_prev);
 
-    if (is_new_window && IS_TRANSIENT(t) && DO_STACK_TRANSIENT_PARENT(t))
+    if (is_new_window && IS_TRANSIENT(t) && DO_STACK_TRANSIENT_PARENT(t) &&
+	!IS_ICONIFIED(t))
     {
       /* now that the new transient is properly positioned in the stack ring,
        * raise/lower it again so that its parent is raised/lowered too */
@@ -715,7 +729,10 @@ static void RaiseOrLowerWindow(
   /* If the window has been raised, make sure the decorations are updated
    * immediately in case we are in a complex function (e.g. raise, unshade). */
   if (!do_lower)
-    DrawDecorations(t, DRAW_ALL, (Scr.Hilite == t), True, None);
+  {
+    XSync(dpy, 0);
+    handle_all_expose();
+  }
 }
 
 /*
@@ -1064,8 +1081,11 @@ int get_layer(FvwmWindow *t)
 
 /* This function recursively finds the transients of the window t and sets their
  * is_in_transient_subtree flag.  If a layer is given, only windows in this
- * layer are checked.  If the layer is < 0, all windows are considered. */
-static void mark_transient_subtree(FvwmWindow *t, int layer, Bool do_lower)
+ * layer are checked.  If the layer is < 0, all windows are considered.
+ */
+void mark_transient_subtree(
+  FvwmWindow *t, int layer, int mark_mode, Bool do_ignore_icons,
+  Bool use_window_group_hint)
 {
   FvwmWindow *s;
   FvwmWindow *start;
@@ -1109,29 +1129,67 @@ static void mark_transient_subtree(FvwmWindow *t, int layer, Bool do_lower)
   }
   /* now loop over the windows and mark the ones we need to move */
   SET_IN_TRANSIENT_SUBTREE(t, 1);
-  is_finished = True;
+  is_finished = False;
   while (!is_finished)
   {
     FvwmWindow *r;
+    FvwmWindow *u;
 
     /* recursively search for all transient windows */
-    is_finished = 1;
+    is_finished = True;
     for (s = start; s != end; s = s->stack_next)
     {
-      if (IS_IN_TRANSIENT_SUBTREE(s) || !IS_TRANSIENT(s))
+      Bool use_group_hint = False;
+
+      if (IS_IN_TRANSIENT_SUBTREE(s))
+	continue;
+      if (DO_ICONIFY_WINDOW_GROUPS(s) && s->wmhints &&
+	  (s->wmhints->flags & WindowGroupHint) &&
+	  (s->wmhints->window_group != None) &&
+	  (s->wmhints->window_group != s->w) &&
+	  (s->wmhints->window_group != Scr.Root))
+      {
+	use_group_hint = True;
+      }
+      if (!IS_TRANSIENT(s) && !use_group_hint)
 	continue;
       r = (FvwmWindow *)s->pscratch;
-      if (r && IS_IN_TRANSIENT_SUBTREE(r) &&
-	  ((do_lower && DO_LOWER_TRANSIENT(r)) ||
-	   (!do_lower && DO_RAISE_TRANSIENT(r))))
+      if (do_ignore_icons && IS_ICONIFIED(r))
+	continue;
+      if (IS_TRANSIENT(s))
       {
-	/* have to move this one too */
-	SET_IN_TRANSIENT_SUBTREE(t, 1);
-	/* need another scan through the list */
-	is_finished = False;
+	if (r && IS_IN_TRANSIENT_SUBTREE(r) &&
+	    ((mark_mode == MARK_ALL) ||
+	     (mark_mode == MARK_LOWER && DO_LOWER_TRANSIENT(r)) ||
+	     (mark_mode == MARK_RAISE && DO_RAISE_TRANSIENT(r))))
+	{
+	  /* have to move this one too */
+	  SET_IN_TRANSIENT_SUBTREE(s, 1);
+	  /* need another scan through the list */
+	  is_finished = False;
+	  continue;
+	}
       }
-    }
-  }
+      if (use_group_hint)
+      {
+	for (u = start; u != end; u = u->stack_next)
+	{
+	  if (u->w == s->wmhints->window_group ||
+	      (u->wmhints && (u->wmhints->flags & WindowGroupHint) &&
+	       u->wmhints->window_group == s->wmhints->window_group))
+	  {
+	    if (IS_IN_TRANSIENT_SUBTREE(u))
+	    {
+	      /* have to move this one too */
+	      SET_IN_TRANSIENT_SUBTREE(s, 1);
+	      /* need another scan through the list */
+	      is_finished = False;
+	    }
+	  }
+	}
+      }
+    } /* for */
+  } /* while */
 
   return;
 }
@@ -1142,7 +1200,8 @@ static int collect_transients_recursive(
   FvwmWindow *s;
   int count = 0;
 
-  mark_transient_subtree(t, layer, do_lower);
+  mark_transient_subtree(
+    t, layer, (do_lower) ? MARK_LOWER : MARK_RAISE, True, False);
   /* now collect the marked windows in a separate list */
   for (s = Scr.FvwmRoot.stack_next; s != &Scr.FvwmRoot; )
   {
