@@ -16,10 +16,15 @@
 #ifndef _STACK_H
 #define _STACK_H
 
+#define DEBUG_STACK_RING 1
+#ifdef DEBUG_STACK_RING
+void verify_stack_ring_consistency(void);
+#endif
 void remove_window_from_stack_ring(FvwmWindow *t);
 void add_window_to_stack_ring_after(FvwmWindow *t, FvwmWindow *add_after_win);
 FvwmWindow *get_next_window_in_stack_ring(FvwmWindow *t);
 FvwmWindow *get_prev_window_in_stack_ring(FvwmWindow *t);
+FvwmWindow *get_transientfor_fvwmwindow(FvwmWindow *t);
 Bool position_new_window_in_stack_ring(FvwmWindow *t, Bool do_lower);
 void RaiseWindow(FvwmWindow *t);
 void LowerWindow(FvwmWindow *t);
@@ -43,6 +48,17 @@ void raise_function(F_CMD_ARGS);
 void lower_function(F_CMD_ARGS);
 void change_layer(F_CMD_ARGS);
 void SetDefaultLayers(F_CMD_ARGS);
+/* This function recursively finds the transients of the window t and sets their
+ * is_in_transient_subtree flag.  If a layer is given, only windows in this
+ * layer are checked.  If the layer is < 0, all windows are considered.
+ */
+#define MARK_RAISE 0
+#define MARK_LOWER 1
+#define MARK_ALL   2
+#define MARK_ALL_LAYERS -1
+void mark_transient_subtree(
+  FvwmWindow *t, int layer, int mark_mode, Bool do_ignore_icons,
+  Bool use_window_group_hint);
 
 
 #endif /* _STACK_H */
