@@ -122,6 +122,13 @@
 #define SFSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(st,x) ((st).flags.common.s.do_grab_focus_when_transient_created = !!(x))
 #define SMSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(st,x) ((st).flag_mask.common.s.do_grab_focus_when_transient_created = !!(x))
 #define SCSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(st,x) ((st).change_mask.common.s.do_grab_focus_when_transient_created = !!(x))
+#define SDO_ICONIFY_WINDOW_GROUPS(sf) ((sf).common.s.do_iconify_window_groups)
+#define SFDO_ICONIFY_WINDOW_GROUPS(st) ((st).flags.common.s.do_iconify_window_groups)
+#define SMDO_ICONIFY_WINDOW_GROUPS(st) ((st).flag_mask.common.s.do_iconify_window_groups)
+#define SCDO_ICONIFY_WINDOW_GROUPS(st) ((st).change_mask.common.s.do_iconify_window_groups)
+#define SFSET_DO_ICONIFY_WINDOW_GROUPS(st,x) ((st).flags.common.s.do_iconify_window_groups = !!(x))
+#define SMSET_DO_ICONIFY_WINDOW_GROUPS(st,x) ((st).flag_mask.common.s.do_iconify_window_groups = !!(x))
+#define SCSET_DO_ICONIFY_WINDOW_GROUPS(st,x) ((st).change_mask.common.s.do_iconify_window_groups = !!(x))
 #define SDO_IGNORE_GNOME_HINTS(sf) ((sf).common.s.do_ignore_gnome_hints)
 #define SFDO_IGNORE_GNOME_HINTS(st) ((st).flags.common.s.do_ignore_gnome_hints)
 #define SMDO_IGNORE_GNOME_HINTS(st) ((st).flag_mask.common.s.do_ignore_gnome_hints)
@@ -129,6 +136,13 @@
 #define SFSET_DO_IGNORE_GNOME_HINTS(st,x) ((st).flags.common.s.do_ignore_gnome_hints = !!(x))
 #define SMSET_DO_IGNORE_GNOME_HINTS(st,x) ((st).flag_mask.common.s.do_ignore_gnome_hints = !!(x))
 #define SCSET_DO_IGNORE_GNOME_HINTS(st,x) ((st).change_mask.common.s.do_ignore_gnome_hints = !!(x))
+#define SDO_IGNORE_ICON_BOXES(sf) ((sf).common.s.do_ignore_icon_boxes)
+#define SFDO_IGNORE_ICON_BOXES(st) ((st).flags.common.s.do_ignore_icon_boxes)
+#define SMDO_IGNORE_ICON_BOXES(st) ((st).flag_mask.common.s.do_ignore_icon_boxes)
+#define SCDO_IGNORE_ICON_BOXES(st) ((st).change_mask.common.s.do_ignore_icon_boxes)
+#define SFSET_DO_IGNORE_ICON_BOXES(st,x) ((st).flags.common.s.do_ignore_icon_boxes = !!(x))
+#define SMSET_DO_IGNORE_ICON_BOXES(st,x) ((st).flag_mask.common.s.do_ignore_icon_boxes = !!(x))
+#define SCSET_DO_IGNORE_ICON_BOXES(st,x) ((st).change_mask.common.s.do_ignore_icon_boxes = !!(x))
 #define SDO_IGNORE_RESTACK(sf) ((sf).common.s.do_ignore_restack)
 #define SFDO_IGNORE_RESTACK(st) ((st).flags.common.s.do_ignore_restack)
 #define SMDO_IGNORE_RESTACK(st) ((st).flag_mask.common.s.do_ignore_restack)
@@ -136,6 +150,13 @@
 #define SFSET_DO_IGNORE_RESTACK(st,x) ((st).flags.common.s.do_ignore_restack = !!(x))
 #define SMSET_DO_IGNORE_RESTACK(st,x) ((st).flag_mask.common.s.do_ignore_restack = !!(x))
 #define SCSET_DO_IGNORE_RESTACK(st,x) ((st).change_mask.common.s.do_ignore_restack = !!(x))
+#define SDO_USE_WINDOW_GROUP_HINT(sf) ((sf).common.s.do_use_window_group_hint)
+#define SFDO_USE_WINDOW_GROUP_HINT(st) ((st).flags.common.s.do_use_window_group_hint)
+#define SMDO_USE_WINDOW_GROUP_HINT(st) ((st).flag_mask.common.s.do_use_window_group_hint)
+#define SCDO_USE_WINDOW_GROUP_HINT(st) ((st).change_mask.common.s.do_use_window_group_hint)
+#define SFSET_DO_USE_WINDOW_GROUP_HINT(st,x) ((st).flags.common.s.do_use_window_group_hint = !!(x))
+#define SMSET_DO_USE_WINDOW_GROUP_HINT(st,x) ((st).flag_mask.common.s.do_use_window_group_hint = !!(x))
+#define SCSET_DO_USE_WINDOW_GROUP_HINT(st,x) ((st).change_mask.common.s.do_use_window_group_hint = !!(x))
 #define SDO_LOWER_TRANSIENT(sf) ((sf).common.s.do_lower_transient)
 #define SFDO_LOWER_TRANSIENT(st) ((st).flags.common.s.do_lower_transient)
 #define SMDO_LOWER_TRANSIENT(st) ((st).flag_mask.common.s.do_lower_transient)
