@@ -187,15 +187,19 @@ static void apply_window_updates(
     rectangle old_g;
 
     setup_frame_size_limits(t, pstyle);
-    gravity_constrain_size(t->hints.win_gravity, t, &(t->frame_g));
     old_g = t->frame_g;
     t->frame_g = t->normal_g;
+    gravity_constrain_size(t->hints.win_gravity, t, &(t->frame_g));
+    t->normal_g = t->frame_g;
     if (IS_MAXIMIZED(t))
     {
       t->frame_g = t->max_g;
-      gravity_constrain_size(t->hints.win_gravity, t, &(t->max_g));
+      gravity_constrain_size(t->hints.win_gravity, t, &(t->frame_g));
+      t->max_g = t->frame_g;
     }
     t->frame_g = old_g;
+    gravity_constrain_size(t->hints.win_gravity, t, &(t->frame_g));
+
     flags->do_setup_frame = True;
     flags->do_redraw_decoration = True;
   }
@@ -275,10 +279,12 @@ static void apply_window_updates(
   {
     setup_frame_attributes(t, pstyle);
   }
-  if (flags->do_update_modules_flags) {
+  if (flags->do_update_modules_flags)
+  {
     BroadcastConfig(M_CONFIGURE_WINDOW,t);
   }
   t->shade_anim_steps = pstyle->shade_anim_steps;
+
   return;
 }
 
@@ -326,6 +332,7 @@ void apply_decor_change(FvwmWindow *tmp_win)
   flags.do_redecorate = True;
   flags.do_update_window_font_height = True;
   apply_window_updates(tmp_win, &flags, &style, Scr.Focus);
+  Scr.flags.do_need_window_update = 1;
 }
 
 /* Check and apply new style to each window if the style has changed. */
