@@ -36,10 +36,13 @@ typedef struct
   unsigned do_update_window_color_hi : 1;
   unsigned do_update_window_font : 1;
   unsigned do_update_window_font_height : 1;
+  unsigned do_update_window_grabs : 1;
   unsigned do_update_modules_flags : 1;
 } update_win;
 
+void destroy_scheduled_windows(void);
 void apply_decor_change(FvwmWindow *tmp_win);
 void flush_window_updates(void);
+void update_styles_func(F_CMD_ARGS);
 
 #endif /* _UPDATE_ */
