@@ -376,16 +376,14 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
     return False;
   }
 
-  if((Scr.ScrollResistance >= 10000)||
-     ((HorWarpSize ==0)&&(VertWarpSize==0)))
+  if (Scr.ScrollResistance >= 10000 || (HorWarpSize == 0 && VertWarpSize==0))
   {
     is_timestamp_valid = False;
     add_time = 0;
     return False;
   }
 
-  /* need to move the viewport */
-  if(Scr.VxMax == 0 && Scr.VyMax == 0)
+  if (Scr.VxMax == 0 && Scr.VyMax == 0)
   {
     is_timestamp_valid = False;
     add_time = 0;
@@ -409,7 +407,7 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 
   do
   {
-    if(XPending(dpy) > 0 &&
+    if (XPending(dpy) > 0 &&
        (XCheckWindowEvent(
 	 dpy, Scr.PanFrameTop.win, LeaveWindowMask, &Event) ||
 	XCheckWindowEvent(
@@ -428,18 +426,22 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
     GetLocationFromEventOrQuery(dpy, Scr.Root, &Event, &x, &y);
     /* check actual pointer location since PanFrames can get buried under
        a window being moved or resized - mab */
-    if(( x >= edge_thickness )&&( x < Scr.MyDisplayWidth-edge_thickness )&&
-       ( y >= edge_thickness )&&( y < Scr.MyDisplayHeight-edge_thickness ))
+    if (x >= edge_thickness && x < Scr.MyDisplayWidth-edge_thickness &&
+        y >= edge_thickness && y < Scr.MyDisplayHeight-edge_thickness)
     {
       is_timestamp_valid = False;
       add_time = 0;
+      discard_window_events(Scr.PanFrameTop.win, LeaveWindowMask);
+      discard_window_events(Scr.PanFrameBottom.win, LeaveWindowMask);
+      discard_window_events(Scr.PanFrameLeft.win, LeaveWindowMask);
+      discard_window_events(Scr.PanFrameRight.win, LeaveWindowMask);
       return False;
     }
     if (!fLoop && is_last_position_valid &&
-	((x - last_x) > MAX_PAGING_MOVE_DISTANCE ||
-	 (x - last_x) < -MAX_PAGING_MOVE_DISTANCE ||
-	 (y - last_y) > MAX_PAGING_MOVE_DISTANCE ||
-	 (y - last_y) < -MAX_PAGING_MOVE_DISTANCE))
+	(x - last_x > MAX_PAGING_MOVE_DISTANCE ||
+	 x - last_x < -MAX_PAGING_MOVE_DISTANCE ||
+	 y - last_y > MAX_PAGING_MOVE_DISTANCE ||
+	 y - last_y < -MAX_PAGING_MOVE_DISTANCE))
     {
       /* The pointer is moving too fast, prevent paging until it slows
        * down. Don't prevent paging when fLoop is set since we can't be
@@ -460,7 +462,9 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 	   lastTimestamp - my_timestamp + add_time < Scr.ScrollResistance);
 
   if (lastTimestamp - my_timestamp + add_time < Scr.ScrollResistance)
+  {
     return False;
+  }
 
   /* Move the viewport */
   /* and/or move the cursor back to the approximate correct location */
@@ -541,16 +545,18 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
     *yt = y - *delta_y;
 
   /* make sure the pointer isn't warped into the panframes */
-  if(*xl < edge_thickness) *xl = edge_thickness;
-  if(*yt < edge_thickness) *yt = edge_thickness;
-  if(*xl >= Scr.MyDisplayWidth - edge_thickness)
+  if (*xl < edge_thickness)
+    *xl = edge_thickness;
+  if (*yt < edge_thickness)
+    *yt = edge_thickness;
+  if (*xl >= Scr.MyDisplayWidth - edge_thickness)
     *xl = Scr.MyDisplayWidth - edge_thickness -1;
-  if(*yt >= Scr.MyDisplayHeight - edge_thickness)
+  if (*yt >= Scr.MyDisplayHeight - edge_thickness)
     *yt = Scr.MyDisplayHeight - edge_thickness -1;
 
   is_timestamp_valid = False;
   add_time = 0;
-  if ((*delta_x == 0)&&(*delta_y == 0))
+  if (*delta_x == 0 && *delta_y == 0)
   {
     return False;
   }
@@ -866,7 +872,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 	 * then the window should essentially be sticky */
 	if(!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) && !IS_STICKY(t))
 	{
-	  if(!(IS_ICON_STICKY(t)))
+	  if (IS_ICONIFIED(t))
 	  {
 	    t->icon_g.x += deltax;
 	    t->icon_xl_loc += deltax;
@@ -914,7 +920,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 	 * then the window should essentially be sticky */
 	if (!(IS_ICONIFIED(t1) && IS_ICON_STICKY(t1)) && !IS_STICKY(t1))
 	{
-	  if (!IS_ICON_STICKY(t1))
+	  if (IS_ICONIFIED(t1))
 	  {
 	    t1->icon_g.x += deltax;
 	    t1->icon_xl_loc += deltax;
@@ -962,6 +968,11 @@ void MoveViewport(int newx, int newy, Bool grab)
     }
   }
   checkPanFrames();
+  if (Scr.Focus)
+  {
+    /* regrab buttons for focused window in case it is now obscured */
+    focus_grab_buttons(Scr.Focus, True);
+  }
 
   /* do this with PanFrames too ??? HEDU */
   while(XCheckTypedEvent(dpy,MotionNotify,&Event))
