@@ -160,12 +160,17 @@
                                ((fw)->flags.common.s.has_mwm_override)
 #define HAS_OVERRIDE_SIZE_HINTS(fw)  \
                                ((fw)->flags.common.s.has_override_size)
+#define DO_ICONIFY_WINDOW_GROUPS(fw)  \
+                               ((fw)->flags.common.s.do_iconify_window_groups)
 #define DO_IGNORE_GNOME_HINTS(fw)  \
                                ((fw)->flags.common.s.do_ignore_gnome_hints)
 #define DO_IGNORE_RESTACK(fw)  \
                                ((fw)->flags.common.s.do_ignore_restack)
-#define IS_FIXED(fw)  \
-                               ((fw)->flags.common.s.is_fixed)
+#define DO_IGNORE_ICON_BOXES(fw)  \
+                               ((fw)->flags.common.s.do_ignore_icon_boxes)
+#define DO_USE_WINDOW_GROUP_HINT(fw)  \
+                             ((fw)->flags.common.s.do_use_window_group_hint)
+#define IS_FIXED(fw)           ((fw)->flags.common.s.is_fixed)
 #define SET_FIXED(fw,x)        (fw)->flags.common.s.is_fixed = !!(x)
 #define SETM_FIXED(fw,x)       (fw)->flag_mask.common.s.is_fixed = !!(x)
 #define HAS_DEPRESSABLE_BORDER(fw) ((fw)->flags.common.s.has_depressable_border)
@@ -296,6 +301,11 @@
           (fw)->flags.is_window_being_moved_opaque = !!(x)
 #define SETM_WINDOW_BEING_MOVED_OPAQUE(fw,x) \
           (fw)->flag_mask.is_window_being_moved_opaque = !!(x)
+#define IS_WINDOW_BORDER_DRAWN(fw)     ((fw)->flags.is_window_border_drawn)
+#define SET_WINDOW_BORDER_DRAWN(fw,x)  \
+          (fw)->flags.is_window_border_drawn = !!(x)
+#define SETM_WINDOW_BORDER_DRAWN(fw,x) \
+          (fw)->flag_mask.is_window_border_drawn = !!(x)
 #define IS_WINDOW_FONT_LOADED(fw)     ((fw)->flags.is_window_font_loaded)
 #define SET_WINDOW_FONT_LOADED(fw,x)  (fw)->flags.is_window_font_loaded = !!(x)
 #define SETM_WINDOW_FONT_LOADED(fw,x) \
