@@ -448,6 +448,7 @@ unsigned int ParseGradient(char *gradient, char **rest, char ***colors_return,
     perc = (int *)safemalloc(sizeof(int) * nsegs);
     for (i = 0; i <= nsegs; i++)
     {
+      s_colors[i] = 0;
       gradient = GetNextToken(gradient, &s_colors[i]);
       if (i < nsegs)
       {
@@ -480,6 +481,24 @@ unsigned int ParseGradient(char *gradient, char **rest, char ***colors_return,
       return 0;
     }
   }
+  for (i = 0; i <= nsegs; i++)
+  {
+    if (s_colors[i] == 0)
+    {
+      int j;
+
+      for (j = 0; j <= nsegs; j++)
+      {
+	if (s_colors[j])
+	  free(s_colors[j]);
+      }
+      free(s_colors);
+      free(perc);
+      if (rest)
+	*rest = gradient;
+      return 0;
+    }
+  }
 
   /* sensible limits */
   if (npixels < 2)
@@ -493,6 +512,7 @@ unsigned int ParseGradient(char *gradient, char **rest, char ***colors_return,
   *nsegs_return = nsegs;
   if (rest)
     *rest = gradient;
+
   return npixels;
 }
 
@@ -566,13 +586,10 @@ Bool CalculateGradientDimensions(
  * a new pixmap of the given depth, width and height is created. If it is not
  * None the gradient is drawn into it. The d_width, d_height, d_x and d_y
  * describe the traget rectangle within the drawable. */
-Drawable CreateGradientPixmap(Display *dpy, Drawable d, GC gc,
-			      int type, int g_width, int g_height,
-			      int ncolors, Pixel *pixels,
-			      Drawable in_drawable,
-			      int d_x, int d_y,
-			      int d_width,
-			      int d_height)
+Drawable CreateGradientPixmap(
+  Display *dpy, Drawable d, GC gc, int type, int g_width, int g_height,
+  int ncolors, Pixel *pixels, Drawable in_drawable, int d_x, int d_y,
+  int d_width, int d_height, XRectangle *rclip)
 {
   Pixmap pixmap = None;
   XImage *image;
@@ -776,8 +793,16 @@ Drawable CreateGradientPixmap(Display *dpy, Drawable d, GC gc,
   xgcv.fill_style = FillSolid;
   xgcv.clip_mask = None;
   XChangeGC(dpy, gc, GCFunction|GCPlaneMask|GCFillStyle|GCClipMask, &xgcv);
+  if (rclip)
+  {
+    XSetClipRectangles(dpy, gc, 0, 0, rclip, 1, Unsorted);
+  }
   /* copy the image to the server */
   XPutImage(dpy, target, gc, image, 0, 0, t_x, t_y, t_width, t_height);
+  if (rclip)
+  {
+    XSetClipMask(dpy, gc, None);
+  }
   XDestroyImage(image);
   return target;
 }
@@ -823,7 +848,7 @@ Pixmap CreateGradientPixmapFromString(Display *dpy, Drawable d, GC gc,
 				  height_return))
     pixmap = CreateGradientPixmap(dpy, d, gc, type, *width_return,
 				  *height_return, ncolors, pixels, None, 0, 0,
-				  0, 0);
+				  0, 0, NULL);
 
   /* if the caller has not asked for the pixels there is probably a leak */
   if (!pixels_return) {
