@@ -152,7 +152,7 @@ static char *CopyToken(char *src, char *dest, char *spaces, int snum,
 			   (snum && strchr(spaces, *src)) ||
 			   (dnum && strchr(delims, *src))))
   {
-    /* Check for qouted text */
+    /* Check for quoted text */
     if (IsQuote(*src))
     {
       char c = *src;
@@ -352,7 +352,7 @@ char *DoGetNextToken(char *indata, char **token, char *spaces, char *delims,
   if (tmptok == NULL)
     *token = NULL;
   else
-    *token = strdup(tmptok);
+    *token = safestrdup(tmptok);
 
   return end;
 }
