@@ -59,6 +59,7 @@
 
 #include "libs/defaults.h"
 #include "libs/fvwmlib.h"
+#include "libs/Grab.h"
 #include "fvwm/fvwm.h"
 #include "libs/Module.h"
 #include "libs/fvwmsignal.h"
@@ -90,6 +91,12 @@
 #define MySendText(a,b,c) SendText((a),(b),(c));
 #endif
 
+#ifdef SHAPE
+int ShapeEventBase = -10000;
+int ShapeErrorBase = -10000;
+Boolean ShapesSupported=False;
+#endif /* SHAPE */
+
 extern int nColorsets;	/* in libs/Colorsets.c */
 
 /* --------------------------- external functions -------------------------- */
@@ -318,8 +325,7 @@ static void DeadPipeCleanup(void)
 
   signal(SIGPIPE, SIG_IGN);/* Xsync may cause SIGPIPE */
 
-  XSync(Dpy,0); /* Wait for thing to settle down a bit */
-  XGrabServer(Dpy); /* We don't want interference right now */
+  MyXGrabServer(Dpy); /* We don't want interference right now */
   while(NextButton(&ub,&b,&button,0))
   {
     swin = SwallowedWindow(b);
@@ -366,8 +372,7 @@ static void DeadPipeCleanup(void)
 #endif
     }
   }
-  XUngrabServer(Dpy); /* We're through */
-  XSync(Dpy,0); /* Let it all die down again so we can catch our X errors... */
+  MyXUngrabServer(Dpy); /* We're through */
 
   /* Hey, we have to free the pictures too! */
   button=-1;
@@ -522,20 +527,27 @@ void SetTransparentBackground(button_info *ub,int w,int h)
   unsigned int depth_return;
   int number, i;
   XFontStruct *font;
+  Region shape_r;
 
+  if (!ShapesSupported)
+    return;
   pmap_mask = XCreatePixmap(Dpy,MyWindow,w,h,1);
-
+  if (pmap_mask == None)
+    return;
+  shape_r = XCreateRegion();
   if (trans_gc == NULL)
   {
     XGCValues gcv;
 
     /* create a GC for doing transparency */
     trans_gc = fvwmlib_XCreateGC(Dpy,pmap_mask,(unsigned long)0,&gcv);
+    if (trans_gc == NULL)
+      return;
   }
-
-  XSetForeground(Dpy,trans_gc,0);
-  XFillRectangle(Dpy,pmap_mask,trans_gc,0,0,w,h);
-  XSetForeground(Dpy,trans_gc,1);
+  XSetClipMask(Dpy, trans_gc, None);
+  XSetForeground(Dpy, trans_gc, 0);
+  XFillRectangle(Dpy, pmap_mask, trans_gc, 0, 0, w, h);
+  XSetForeground(Dpy, trans_gc, 1);
 
   /*
    * if button has an icon, draw a rect with the same size as the icon
@@ -543,15 +555,15 @@ void SetTransparentBackground(button_info *ub,int w,int h)
    * else draw a rect with the same size as the button.
    */
 
-  i=-1;
-  while(NextButton(&ub,&b,&i,0))
+  i = -1;
+  while (NextButton(&ub,&b,&i,0))
   {
+    number=buttonNum(b);
     if (b->flags&b_Icon &&
 	XGetGeometry(Dpy,b->IconWin,&root_return,&x_return,&y_return,
 		     &width_return,&height_return,
 		     &border_width_return,&depth_return))
     {
-      number=buttonNum(b);
       if (b->icon->mask == None)
       {
 	XFillRectangle(Dpy,pmap_mask,trans_gc,x_return, y_return,
@@ -563,9 +575,57 @@ void SetTransparentBackground(button_info *ub,int w,int h)
 		  b->icon->width,b->icon->height,x_return,y_return);
       }
     }
+    else if (buttonSwallowCount(b) == 3 && (b->flags & b_Swallow))
+    {
+      Window swin = SwallowedWindow(b);
+      XRectangle *r;
+      int count;
+      int ordering;
+      XRectangle s;
+      Bool got_geometry = False;
+
+      if (XGetGeometry(
+	    Dpy, swin, &root_return, &x_return, &y_return, &width_return,
+	    &height_return, &border_width_return, &depth_return))
+      {
+	got_geometry = True;
+	s.x = x_return;
+	s.y = y_return;
+	s.width = width_return;
+	s.height = height_return;
+      }
+      else
+      {
+	s.x = buttonXPos(b, number);
+	s.y = buttonYPos(b, number);
+	s.width = buttonWidth(b);
+	s.height = buttonHeight(b);
+      }
+
+      /* get set of shape rectangles - this is all very inefficient, but I do
+       * not see another way to do it properly */
+      if (got_geometry &&
+	  (r = XShapeGetRectangles(
+	    Dpy, swin, ShapeBounding, &count, &ordering)))
+      {
+	int i;
+
+	/* merge the rectangles with the shape region */
+	for (i = 0; i < count ; i++)
+	{
+	  r[i].x += s.x;
+	  r[i].y += s.y;
+	  XUnionRectWithRegion(&(r[i]), shape_r, shape_r);
+	}
+	XFree(r);
+      }
+      else
+      {
+	XUnionRectWithRegion(&s, shape_r, shape_r);
+      }
+    }
     else
     {
-      number=buttonNum(b);
       XFillRectangle(Dpy,pmap_mask,trans_gc,buttonXPos(b,number),
 		     buttonYPos(b,number),
 		     buttonWidth(b),buttonHeight(b));
@@ -579,7 +639,13 @@ void SetTransparentBackground(button_info *ub,int w,int h)
       DrawTitle(b,pmap_mask,trans_gc);
     }
   }
-  XShapeCombineMask(Dpy,MyWindow,ShapeBounding,0,0,pmap_mask,ShapeSet);
+  XSetRegion(Dpy, trans_gc, shape_r);
+  XFillRectangle(Dpy, pmap_mask, trans_gc, 0, 0, w, h);
+  XShapeCombineMask(Dpy, MyWindow, ShapeBounding, 0, 0, pmap_mask, ShapeSet);
+  XFreePixmap(Dpy, pmap_mask);
+  XDestroyRegion(shape_r);
+
+  return;
 }
 #endif
 
@@ -709,12 +775,12 @@ int main(int argc, char **argv)
     else if (!has_name) /* There is a naming argument here! */
     {
       free(MyName);
-      MyName=strdup(argv[i]);
+      MyName=safestrdup(argv[i]);
       has_name = 1;
     }
     else if (!has_file) /* There is a config file here! */
     {
-      config_file=strdup(argv[i]);
+      config_file=safestrdup(argv[i]);
       has_file = 1;
     }
   }
@@ -760,6 +826,10 @@ int main(int argc, char **argv)
   UberButton->title   = MyName;
   UberButton->swallow = 1; /* the panel is shown */
 
+#ifdef SHAPE
+  ShapesSupported = XShapeQueryExtension(Dpy, &ShapeEventBase, &ShapeErrorBase);
+#endif /* SHAPE */
+
   /* parse module options */
   ParseConfiguration(UberButton);
   /* we can't set the size if it was specified in pixels per button here;
@@ -842,8 +912,11 @@ int main(int argc, char **argv)
     ConfigureIconWindow(b);
 
 #ifdef SHAPE
-  if(UberButton->c->flags&b_TransBack)
-    SetTransparentBackground(UberButton,Width,Height);
+  if (ShapesSupported)
+  {
+    if (UberButton->c->flags&b_TransBack)
+      SetTransparentBackground(UberButton,Width,Height);
+  }
 #endif
 
   i=-1;
@@ -905,6 +978,23 @@ void Loop(void)
   {
     if(My_XNextEvent(Dpy,&Event))
     {
+#ifdef SHAPE
+      if (ShapesSupported)
+      {
+	if (Event.type == ShapeEventBase + ShapeNotify)
+	{
+	  XShapeEvent *sev = (XShapeEvent *) &Event;
+
+	  if (sev->kind != ShapeBounding)
+	    return;
+	  if (UberButton->c->flags & b_TransBack)
+	  {
+	    SetTransparentBackground(UberButton, Width, Height);
+	  }
+	  continue;
+	}
+      }
+#endif
       switch(Event.type)
       {
       case Expose:
@@ -975,6 +1065,10 @@ void Loop(void)
 	  SetButtonSize(UberButton,Width,Height);
 	  button=-1;
 	  ub=UberButton;
+	  if (UberButton->c->flags & b_TransBack)
+	  {
+	    SetTransparentBackground(UberButton, Width, Height);
+	  }
 	  for (i = 0; i < nColorsets; i++)
 	    change_colorset(i);
 	  while(NextButton(&ub,&b,&button,0))
@@ -990,6 +1084,13 @@ void Loop(void)
 	  break;	                /* fall through to ButtonPress */
 
       case ButtonPress:
+	if (CurrentButton)
+	{
+	  b = CurrentButton;
+	  CurrentButton = 0;
+	  RedrawButton(b, 0);
+	  break;
+	}
 	CurrentButton = b =
 	  select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
 
@@ -1015,9 +1116,8 @@ void Loop(void)
 
       case KeyRelease:
       case ButtonRelease:
-	b=select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
+	b = select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
 	act = GetButtonAction(b,Event.xbutton.button);
-
 	if (b && !act && (b->flags & b_Panel))
 	{
 	  HandlePanelPress(b);
@@ -1038,14 +1138,14 @@ void Loop(void)
 	{
 	  if (!act)
 	    act=GetButtonAction(b,0);
-	  if(b && b==CurrentButton && act)
+	  if (b && b == CurrentButton && act)
 	  {
 	    if(strncasecmp(act,"Exec",4)==0)
 	    {
 
 	      /* Look for Exec "identifier", in which case the button
 		 stays down until window "identifier" materializes */
-	      i=4;
+	      i = 4;
 	      while(isspace((unsigned char)act[i]))
 		i++;
 	      if(act[i] == '"')
@@ -1120,10 +1220,10 @@ void Loop(void)
 	  free(act);
 	  act = NULL;
 	}
-	b=CurrentButton;
+	b = CurrentButton;
 	CurrentButton=NULL;
-	if(b)
-	  RedrawButton(b,0);
+	if (b)
+	  RedrawButton(b, 0);
 	break;
 
       case ClientMessage:
@@ -1168,6 +1268,15 @@ void Loop(void)
 	      if(!XGetWMNormalHints(Dpy,swin,b->hints,&supp))
 		b->hints->flags = 0;
 	      MakeButton(b);
+#ifdef SHAPE
+	      if (ShapesSupported)
+	      {
+		if (UberButton->c->flags & b_TransBack)
+		{
+		  SetTransparentBackground(UberButton, Width, Height);
+		}
+	      }
+#endif
 	    }
 	    RedrawButton(b,1);
 	  }
@@ -1690,7 +1799,7 @@ static void HandlePanelPress(button_info *b)
 
   /* We're sure that the window is mapped and decorated now. Find the top level
    * ancestor now. */
-  XGrabServer(Dpy);
+  MyXGrabServer(Dpy);
   lb = 0;
   tb = 0;
   rb = 0;
@@ -1725,7 +1834,7 @@ static void HandlePanelPress(button_info *b)
   /* now find the source and destination positions for the slide operation */
   GetPanelGeometry(is_mapped, b, lb, tb, rb, bb, &x1, &y1, &w1, &h1);
   GetPanelGeometry(!is_mapped, b, lb, tb, rb, bb, &x2, &y2, &w2, &h2);
-  XUngrabServer(Dpy);
+  MyXUngrabServer(Dpy);
 
   /* to force fvwm to map the window where we want */
   if (is_mapped)
@@ -1749,16 +1858,16 @@ static void HandlePanelPress(button_info *b)
 	      x2, y2, w2, h2,
 	      steps, b->slide_delay_ms, NULL, b->panel_flags.smooth,
 	      (b->swallow&b_NoHints) ? 0 : 1);
+  if (is_mapped)
+  {
+    XUnmapWindow(Dpy, b->PanelWin);
+  }
   XSync(Dpy, 0);
   /* Give fvwm a chance to update the window.  Otherwise the window may end up
    * too small.  This doesn't prevent this completely, but makes it much less
    * likely. */
   usleep(250000);
 
-  if (is_mapped)
-  {
-    XUnmapWindow(Dpy, b->PanelWin);
-  }
   RedrawButton(b, 1);
 
   return;
@@ -1885,8 +1994,8 @@ void CreateUberButtonWindow(button_info *ub,int maxx,int maxy)
 #endif
   XSetWMProtocols(Dpy,MyWindow,&_XA_WM_DEL_WIN,1);
 
-  myclasshints.res_name=strdup(MyName);
-  myclasshints.res_class=strdup("FvwmButtons");
+  myclasshints.res_name=safestrdup(MyName);
+  myclasshints.res_class=safestrdup("FvwmButtons");
 
   {
     XTextProperty mynametext;
@@ -2657,7 +2766,7 @@ void swallow(unsigned long *body)
       {
 	/* "Swallow" the window! Place it in the void so we don't see it
 	 * until it's MoveResize'd */
-	XGrabServer(Dpy);
+	MyXGrabServer(Dpy);
 	XSync(Dpy, 0);
 	do_allow_bad_access = True;
 	XSelectInput(Dpy,swin,SW_EVENTS);
@@ -2672,13 +2781,12 @@ void swallow(unsigned long *body)
 	  b->swallow&=~b_Count;
 	  b->swallow|=1;
 	  b->flags|=b_Hangon;
-	  XUngrabServer(Dpy);
+	  MyXUngrabServer(Dpy);
 	  return;
 	}
 	/*error checking*/
 	XReparentWindow(Dpy,swin,MyWindow,-32768,-32768);
-	XUngrabServer(Dpy);
-	XSync(Dpy, 0);
+	MyXUngrabServer(Dpy);
 	b->swallow&=~b_Count;
 	b->swallow|=3;
 	if(buttonSwallow(b)&b_UseTitle)
@@ -2711,6 +2819,16 @@ void swallow(unsigned long *body)
 	  XSync(Dpy, 0);
 	  change_swallowed_window_colorset(b, True);
 	}
+#ifdef SHAPE
+	if (ShapesSupported)
+	{
+	  if (UberButton->c->flags & b_TransBack)
+	  {
+	    SetTransparentBackground(UberButton, Width, Height);
+	    XShapeSelectInput(Dpy, swin, ShapeNotifyMask);
+	  }
+	}
+#endif
 	RedrawButton(b,1);
       }
       else /* (b->flags & b_Panel) */
