@@ -181,7 +181,8 @@ void MakeButton(button_info *b)
 /**
 *** RedrawButton()
 *** Writes out title, if any, and displays the bevel right, by calling
-*** RelieveButton. If clean is nonzero, also clears background.
+*** RelieveButton. If clean is nonzero, also clears background. If clean is 2,
+*** forces clearing the background (used in case a panel has lost its window).
 **/
 void RedrawButton(button_info *b,int clean)
 {
@@ -250,8 +251,10 @@ void RedrawButton(button_info *b,int clean)
     if(n<4)
     {
       k=sscanf(&b->action[n][4],"%d%d",&i,&j);
-      if(k==2 && i==0 && j==new_desk)
+      if (k == 2 && i == 0 && j == new_desk)
+      {
 	rev=1;
+      }
     }
   }
 
@@ -318,6 +321,10 @@ void RedrawButton(button_info *b,int clean)
     {
       XFillRectangle(Dpy,MyWindow,NormalGC,x+f,y+f,BW-2*f,BH-2*f);
     }
+    else if (clean == 2)
+    {
+      XClearArea(Dpy, MyWindow, x+f, y+f, BW-2*f, BH-2*f, False);
+    }
   }
 
   /* ----------------------------------------------------------------------- */
