@@ -53,8 +53,11 @@
 #include "libs/Module.h"
 #include "libs/Picture.h"
 #include "libs/Colorset.h"
+#include "libs/fvwmsignal.h"
 #include "FvwmIdent.h"
 
+static RETSIGTYPE TerminateHandler(int);
+
 static char *MyName;
 static fd_set_size_t fd_width;
 static int fd[2];
@@ -136,8 +139,51 @@ int main(int argc, char **argv)
       exit(1);
     }
 
-  /* Dead pipe == dead fvwm */
-  signal (SIGPIPE, DeadPipe);
+#ifdef HAVE_SIGACTION
+  {
+    struct sigaction  sigact;
+
+    sigemptyset(&sigact.sa_mask);
+    sigaddset(&sigact.sa_mask, SIGPIPE);
+    sigaddset(&sigact.sa_mask, SIGTERM);
+    sigaddset(&sigact.sa_mask, SIGQUIT);
+    sigaddset(&sigact.sa_mask, SIGINT);
+    sigaddset(&sigact.sa_mask, SIGHUP);
+# ifdef SA_INTERRUPT
+    sigact.sa_flags = SA_INTERRUPT;
+# else
+    sigact.sa_flags = 0;
+# endif
+    sigact.sa_handler = TerminateHandler;
+
+    sigaction(SIGPIPE, &sigact, NULL);
+    sigaction(SIGTERM, &sigact, NULL);
+    sigaction(SIGQUIT, &sigact, NULL);
+    sigaction(SIGINT,  &sigact, NULL);
+    sigaction(SIGHUP,  &sigact, NULL);
+  }
+#else
+  /* We don't have sigaction(), so fall back to less robust methods.  */
+#ifdef USE_BSD_SIGNALS
+  fvwmSetSignalMask( sigmask(SIGPIPE) |
+                     sigmask(SIGTERM) |
+                     sigmask(SIGQUIT) |
+                     sigmask(SIGINT) |
+                     sigmask(SIGHUP) );
+#endif
+  signal(SIGPIPE, TerminateHandler);
+  signal(SIGTERM, TerminateHandler);
+  signal(SIGQUIT, TerminateHandler);
+  signal(SIGINT,  TerminateHandler);
+  signal(SIGHUP,  TerminateHandler);
+#ifdef HAVE_SIGINTERRUPT
+  siginterrupt(SIGPIPE, 1);
+  siginterrupt(SIGTERM, 1);
+  siginterrupt(SIGQUIT, 1);
+  siginterrupt(SIGINT, 1);
+  siginterrupt(SIGHUP, 1);
+#endif
+#endif
 
   fd[0] = atoi(argv[1]);
   fd[1] = atoi(argv[2]);
@@ -222,13 +268,14 @@ int main(int argc, char **argv)
  *************************************************************************/
 void Loop(int *fd)
 {
-    while (1) {
-	FvwmPacket* packet = ReadFvwmPacket(fd[1]);
-	if ( packet == NULL )
-	    exit(0);
-	else
-	    process_message( packet->type, packet->body );
-    }
+  while (1)
+  {
+    FvwmPacket* packet = ReadFvwmPacket(fd[1]);
+    if ( packet == NULL )
+      exit(0);
+    else
+      process_message( packet->type, packet->body );
+  }
 }
 
 
@@ -272,12 +319,14 @@ void process_message(unsigned long type,unsigned long *body)
 
 /***********************************************************************
  *
- * Detected a broken pipe - time to exit
+ *  Procedure:
+ *	SIGPIPE handler - SIGPIPE means fvwm is dying
  *
- **********************************************************************/
-void DeadPipe(int nonsense)
+ ***********************************************************************/
+static RETSIGTYPE
+TerminateHandler(int sig)
 {
-  exit(0);
+  fvwmSetTerminate(sig);
 }
 
 /***********************************************************************
@@ -312,7 +361,6 @@ void list_configure(unsigned long *body)
       target.gravity = cfgpacket->hints_win_gravity;
       found = 1;
     }
-
 }
 
 /*************************************************************************
@@ -337,7 +385,7 @@ void list_icon_name(unsigned long *body)
 {
   if((app_win == (Window)body[1])||(app_win == (Window)body[0]))
     {
-      strncat(target.icon_name,(char *)&body[3],255);
+      strncpy(target.icon_name,(char *)&body[3],255);
     }
 }
 
@@ -351,7 +399,7 @@ void list_class(unsigned long *body)
 {
   if((app_win == (Window)body[1])||(app_win == (Window)body[0]))
     {
-      strncat(target.class,(char *)&body[3],255);
+      strncpy(target.class,(char *)&body[3],255);
     }
 }
 
@@ -365,7 +413,7 @@ void list_res_name(unsigned long *body)
 {
   if((app_win == (Window)body[1])||(app_win == (Window)body[0]))
     {
-      strncat(target.res,(char *)&body[3],255);
+      strncpy(target.res,(char *)&body[3],255);
     }
 }
 
@@ -495,7 +543,7 @@ void list_end(void)
   gcm = GCForeground|GCFont;
   gcv.foreground = fore_pix;
   gcv.font = font->fid;
-  gc = XCreateGC(dpy, main_win, gcm, &gcv);
+  gc = fvwmlib_XCreateGC(dpy, main_win, gcm, &gcv);
 
   if (colorset >= 0)
     SetWindowBackground(dpy, main_win, mysizehints.width, mysizehints.height,
@@ -526,7 +574,7 @@ void list_end(void)
 	  is_key_pressed = 1;
 	  break;
         case ButtonPress:
-	  is_button_pressed = 1;
+	  is_button_pressed = Event.xbutton.button;
 	  break;
         case KeyRelease:
 	  if (is_key_pressed)
@@ -535,9 +583,27 @@ void list_end(void)
 	    break;
         case ButtonRelease:
 	  if (is_button_pressed)
-	    exit(0);
-	  else
-	    break;
+	  {
+	    if (is_button_pressed == 2 && Event.xbutton.button == 2)
+	    {
+	      /* select a new window when button 2 is pressed */
+	      fvwmlib_get_target_window(dpy, screen, MyName, &app_win, True);
+	      SetMessageMask(
+		fd, M_CONFIGURE_WINDOW | M_WINDOW_NAME | M_ICON_NAME
+		| M_RES_CLASS | M_RES_NAME | M_END_WINDOWLIST | M_CONFIG_INFO
+		| M_END_CONFIG_INFO | M_SENDCONFIG);
+	      SendInfo(fd,"Send_WindowList",0);
+	      XDestroyWindow(dpy, main_win);
+	      DestroyList();
+	      found = 0;
+	      return;
+	    }
+	    else
+	    {
+	      exit(0);
+	    }
+	  }
+	  break;
         case ClientMessage:
 	  if (Event.xclient.format==32 && Event.xclient.data.l[0]==wm_del_win)
 	    exit(0);
@@ -577,6 +643,8 @@ void list_end(void)
       char *tline, *token;
 
       packet = ReadFvwmPacket(fd[1]);
+      if (packet == NULL)
+	exit(0);
       if (colorset >= 0 && packet && packet->type == M_CONFIG_INFO) {
 	tline = (char*)&(packet->body[3]);
 	tline = GetNextToken(tline, &token);
@@ -665,6 +733,19 @@ void change_window_name(char *str)
 }
 
 
+void DestroyList(void)
+{
+  struct Item *t;
+  struct Item *tmp;
+
+  for (t = itemlistRoot; t; t = tmp)
+  {
+    tmp = t->next;
+    free(t);
+  }
+  itemlistRoot = NULL;
+}
+
 /**************************************************************************
 *
 * Add s1(string at first column) and s2(string at second column) to itemlist
