@@ -137,7 +137,7 @@ extern void ExitPager(void);
 static Pixmap default_pixmap = None;
 
 /* discard certain events on a window */
-static void discard_events(long event_type, Window w)
+static void discard_events(long event_type, Window w, XEvent *last_ev)
 {
   XEvent e;
 
@@ -145,6 +145,10 @@ static void discard_events(long event_type, Window w)
   while (XCheckTypedWindowEvent(dpy, w, event_type, &e))
   {
     /* do nothing */
+    if (last_ev)
+    {
+      memcpy(last_ev, &e, sizeof(XEvent));
+    }
   }
 
   return;
@@ -356,11 +360,8 @@ void initialize_pager(void)
       exit(1);
     }
   };
-  if(uselabel)
-    label_h = font->ascent + font->descent+2;
-  else
-    label_h = 0;
 #endif
+  label_h = (uselabel) ? font->ascent + font->descent + 2 : 0;
 
 #ifdef SHAPE
   /* Check that shape extension exists. */
@@ -403,7 +404,8 @@ void initialize_pager(void)
   back_pix = GetColor(PagerBack);
   hi_pix = GetColor(HilightC);
 
-  if (windowcolorset >= 0) {
+  if (windowcolorset >= 0)
+  {
     win_back_pix = Colorset[windowcolorset].bg;
     win_fore_pix = Colorset[windowcolorset].fg;
     win_pix_set = True;
@@ -482,20 +484,29 @@ void initialize_pager(void)
     if (Rows*Columns < ndesks)
       Rows++;
   }
-  if(window_w > 0)
+
+  sizehints.width_inc = Columns*(n+1);
+  sizehints.height_inc = Rows*(m+1);
+  sizehints.base_width = Columns * n + Columns - 1;
+  sizehints.base_height = Rows * (m + label_h + 1) - 1;
+  if (window_w > 0)
   {
-    window_w = ((window_w - n) / (n + 1)) * (n + 1) + n;
-    Scr.VScale = Columns * Scr.VWidth / (window_w - Columns + 1 - Columns * n);
+    window_w = (window_w - sizehints.base_width) / sizehints.width_inc;
+    window_w = window_w * sizehints.width_inc + sizehints.base_width;
   }
-  if(window_h > 0)
+  else
   {
-    window_h = ((window_h - m) / (m + 1)) * (m + 1) + m;
-    Scr.VScale = Rows * Scr.VHeight /(window_h + 2 - Rows * (label_h - m - 1));
-  }
-  if(window_w <= 0)
     window_w = Columns * (Scr.VWidth / Scr.VScale + n) + Columns - 1;
-  if(window_h <= 0)
-    window_h = Rows * (Scr.VHeight / Scr.VScale + m + label_h + 1) - 2;
+  }
+  if (window_h > 0)
+  {
+    window_h = (window_h - sizehints.base_height) / sizehints.height_inc;
+    window_h = window_h * sizehints.height_inc + sizehints.base_height;
+  }
+  else
+  {
+    window_h = Rows * (Scr.VHeight / Scr.VScale + m + label_h + 1) - 1;
+  }
 
   if (is_transient)
   {
@@ -510,13 +521,12 @@ void initialize_pager(void)
       yneg = 1;
     }
   }
-  if(xneg)
+  if (xneg)
   {
     sizehints.win_gravity = NorthEastGravity;
     window_x = Scr.MyDisplayWidth - window_w + window_x;
   }
-
-  if(yneg)
+  if (yneg)
   {
     window_y = Scr.MyDisplayHeight - window_h + window_y;
     if(sizehints.win_gravity == NorthEastGravity)
@@ -524,6 +534,10 @@ void initialize_pager(void)
     else
       sizehints.win_gravity = SouthWestGravity;
   }
+  sizehints.width = window_w;
+  sizehints.height = window_h;
+  sizehints.x = window_x;
+  sizehints.y = window_y;
 
   if(usposition)
     sizehints.flags |= USPosition;
@@ -533,14 +547,6 @@ void initialize_pager(void)
   attributes.border_pixel = 0;
   attributes.colormap = Pcmap;
   attributes.event_mask = (StructureNotifyMask);
-  sizehints.width = window_w;
-  sizehints.height = window_h;
-  sizehints.x = window_x;
-  sizehints.y = window_y;
-  sizehints.width_inc = Columns*(n+1);
-  sizehints.height_inc = Rows*(m+1);
-  sizehints.base_width = Columns * n + Columns - 1;
-  sizehints.base_height = Rows*(m + label_h+1) - 1;
 
   /* destroy the temp window first, don't worry if it's the Root */
   if (Scr.Pager_w != Scr.Root)
@@ -726,7 +732,7 @@ void initialize_pager(void)
 				     CopyFromParent, valuemask, &attributes);
     attributes.event_mask = (ExposureMask | ButtonReleaseMask |
 			     ButtonPressMask |ButtonMotionMask);
-    desk_h = window_h - label_h;
+    desk_h = (window_h - Rows * label_h - Rows + 1) / Rows;
 
     valuemask &= ~(CWBackPixel);
 
@@ -1014,10 +1020,10 @@ void DispatchEvent(XEvent *Event)
         UnmapBalloonWindow();
       break;
     case ConfigureNotify:
+      discard_events(ConfigureNotify, Event->xconfigure.window, Event);
       w = Event->xconfigure.window;
       ReConfigure();
-      discard_events(ConfigureNotify, w);
-      discard_events(Expose, w);
+      discard_events(Expose, w, NULL);
       break;
     case Expose:
       HandleExpose(Event, False);
@@ -1253,7 +1259,7 @@ void HandleExpose(XEvent *Event, Bool redraw_subwindows)
     }
   }
 
-  discard_events(Expose, Event->xany.window);
+  discard_events(Expose, Event->xany.window, NULL);
 }
 
 
@@ -1278,21 +1284,27 @@ void ReConfigure(void)
   m1 = Scr.Vy / Scr.MyDisplayHeight;
   n = Scr.VxMax / Scr.MyDisplayWidth;
   m = Scr.VyMax / Scr.MyDisplayHeight;
-  if (window_w > 0)
-    window_w = ((window_w - n)/(n+1))*(n+1)+n;
-  if (window_h > 0)
-    window_h = ((window_h - m)/(m+1))*(m+1)+m;
-  desk_w = (window_w - Columns + 1)/Columns;
-  desk_h = (window_h - Rows*label_h - Rows + 1)/Rows;
-  w = (desk_w - n)/(n+1);
-  h = (desk_h - m)/(m+1);
 
-  sizehints.width_inc = Columns*(n+1);
-  sizehints.height_inc = Rows*(m+1);
+  sizehints.width_inc = Columns * (n + 1);
+  sizehints.height_inc = Rows * (m + 1);
   sizehints.base_width = Columns * n + Columns - 1;
   sizehints.base_height = Rows*(m + label_h+1) - 1;
   sizehints.min_width = sizehints.base_width;
   sizehints.min_height = sizehints.base_height;
+  if (window_w > 0)
+  {
+    window_w = (window_w - sizehints.base_width) / sizehints.width_inc;
+    window_w = window_w * sizehints.width_inc + sizehints.base_width;
+  }
+  if (window_h > 0)
+  {
+    window_h = (window_h - sizehints.base_height) / sizehints.height_inc;
+    window_h = window_h * sizehints.height_inc + sizehints.base_height;
+  }
+  desk_w = (window_w - Columns + 1) / Columns;
+  desk_h = (window_h - Rows * label_h - Rows + 1) / Rows;
+  w = (desk_w - n)/(n+1);
+  h = (desk_h - m)/(m+1);
 
   XSetWMNormalHints(dpy,Scr.Pager_w,&sizehints);
 
@@ -1356,7 +1368,6 @@ void MovePage(Bool is_new_desk)
   char str[100],*sptr;
   static int icon_desk_shown = -1000;
 
-XSynchronize(dpy, 1);
   Wait = 0;
   n1 = Scr.Vx/Scr.MyDisplayWidth;
   m1 = Scr.Vy/Scr.MyDisplayHeight;
@@ -1375,7 +1386,9 @@ XSynchronize(dpy, 1);
 	XLowerWindow(dpy,Desks[i].CPagerWin);
       }
       else
+      {
 	XMoveWindow(dpy, Desks[i].CPagerWin, -1000,-1000);
+      }
     }
   }
   DrawIconGrid(1);
@@ -2103,6 +2116,10 @@ void MoveWindow(XEvent *Event)
   wy = (Scr.Vy + t->y) * (desk_h - m) / Scr.VHeight + m1;
   wx1 = wx + (desk_w + 1) * (NewDesk % Columns);
   wy1 = wy + label_h + (desk_h + label_h + 1) * (NewDesk / Columns);
+  if (LabelsBelow)
+  {
+    wy1 -= label_h;
+  }
 
   XReparentWindow(dpy, t->PagerView, Scr.Pager_w, wx1, wy1);
   XRaiseWindow(dpy,t->PagerView);
@@ -2770,7 +2787,7 @@ static void InsertExpand(char **dest, char *s)
    -- disching@fmi.uni-passau.de */
 char *GetBalloonLabel(const PagerWindow *pw,const char *fmt)
 {
-  char *dest = strdup("");
+  char *dest = safestrdup("");
   const char *pos = fmt;
   char buffer[2];
 
